#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base classes for custom management commands.
"""
from __future__ import unicode_literals
import os
import re
from django.utils import timezone
from django.utils.termcolors import colorize
from calaccess_raw import get_data_directory

# Commands
from django.core.management.base import BaseCommand
from django.core.management import CommandError

# Models
from calaccess_raw.models import RawDataVersion
from calaccess_processed.models import ProcessedDataVersion

# Logging
import logging
logger = logging.getLogger(__name__)


class CalAccessCommand(BaseCommand):
    """
    Base class for all custom CalAccess-related management commands.
    """
    def handle(self, *args, **options):
        """
        Sets options common to all commands.

        Any command subclassing this object should implement its own
        handle method, as is standard in Django, and run this method
        via a super call to inherit its functionality.
        """
        # Set global options
        self.verbosity = options.get("verbosity")
        self.no_color = options.get("no_color")

        # Start the clock
        self.start_datetime = timezone.now()

        # set up processed data directory
        self.data_dir = get_data_directory()
        self.processed_data_dir = os.path.join(
            self.data_dir,
            'processed',
        )
        if not os.path.exists(self.processed_data_dir):
            # make the processed data director
            os.makedirs(self.processed_data_dir)

    def get_or_create_processed_version(self):
        """
        Get or create the current processed version.

        Return a tuple (ProcessedDataVersion object, created), where
        created is a boolean specifying whether a version was created.
        """
        # get the latest raw data version
        try:
            latest_raw_version = RawDataVersion.objects.latest(
                'release_datetime',
            )
        except RawDataVersion.DoesNotExist:
            raise CommandError(
                'No raw CAL-ACCESS data loaded (run `python manage.py '
                'updatecalaccessrawdata`).'
            )

        # check if latest raw version update completed
        if latest_raw_version.update_stalled:
            msg_tmp = 'Update to raw version released at %s did not complete'
            raise CommandError(
                msg_tmp % latest_raw_version.release_datetime.ctime()
            )

        return ProcessedDataVersion.objects.get_or_create(
            raw_version=latest_raw_version,
        )

    def header(self, string):
        """
        Writes out a string to stdout formatted to look like a header.
        """
        logger.debug(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="cyan", opts=("bold",))
        self.stdout.write(string)

    def log(self, string):
        """
        Writes out a string to stdout formatted to look like a standard line.
        """
        logger.debug(string)
        if not getattr(self, 'no_color', None):
            string = colorize("%s" % string, fg="white")
        self.stdout.write(string)

    def success(self, string):
        """
        Writes out a string to stdout formatted green to communicate success.
        """
        logger.debug(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="green")
        self.stdout.write(string)

    def warn(self, string):
        """
        Writes string to stdout formatted yellow to communicate a warning.
        """
        logger.warning(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="yellow")
        self.stdout.write(string)

    def failure(self, string):
        """
        Writes string to stdout formatted red to communicate failure.
        """
        logger.error(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="red")
        self.stdout.write(string)

    def duration(self):
        """
        Calculates how long command has been running and writes it to stdout.
        """
        duration = timezone.now() - self.start_datetime
        self.stdout.write('Duration: {}'.format(str(duration)))
        logger.debug('Duration: {}'.format(str(duration)))

    def __str__(self):
        return re.sub(r'(.+\.)*', '', self.__class__.__module__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals
from opencivicdata.elections.models import (
    CandidateContest,
    CandidateContestPost,
    CandidateContestSource
)
from calaccess_processed.proxies import OCDProxyModelMixin
from calaccess_processed.managers import BulkLoadSQLManager
from calaccess_processed_elections.managers import OCDCandidateContestManager


class OCDCandidateContestProxy(CandidateContest, OCDProxyModelMixin):
    """
    A proxy on the OCD CandidateContest model with helper methods.
    """
    objects = OCDCandidateContestManager()

    copy_to_fields = (
        ('id',),
        ('name',),
        ('division_id',),
        ('election_id',),
        ('party_id',),
        ('previous_term_unexpired',),
        ('number_elected',),
        ('runoff_for_contest_id',),
        ('created_at',),
        ('updated_at',),
        ('extras',),
        ('locked_fields',),
    )

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    def get_parent(self):
        """
        Returns the undecided contest that preceeded runoff_contest.

        Returns None if it can't be found.
        """
        # Get the contest's post (should only ever be one per contest)
        post = self.posts.all()[0].post

        # Then try getting the most recent contest for the same post
        # that preceeds the runoff contest
        try:
            return CandidateContest.objects.filter(
                posts__post=post,
                election__date__lt=self.election.date,
            ).latest('election__date')
        except CandidateContest.DoesNotExist:
            return None


class OCDCandidateContestPostProxy(CandidateContestPost, OCDProxyModelMixin):
    """
    A proxy on the OCD CandidateContestPost model.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True


class OCDCandidateContestSourceProxy(CandidateContestSource, OCDProxyModelMixin):
    """
    A proxy on the OCD CandidateContestSource model.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Mixins for proxies of models from calaccess_scraped.
"""
from __future__ import unicode_literals
import re
from datetime import date


class ScrapedElectionProxyMixin(object):
    """
    Mixin with properties and methods shared by all scraped Election proxy models.
    """
    def get_or_create_ocd_election(self):
        """
        Get the OCD Election for the scraped election instance, or create a new one.

        Side effects of getting:
        * The scraped election's type will be appended to the 'calaccess_election_type'
        of the OCD Election's extras (if not already included).
        * The scraped election's scraped_id (if it exists) will be appended to the OCD
        Election's idenfitiers.

        Returns a tuple (Election object, created), where created is a boolean
        specifying whether a Election was created.
        """
        from ..opencivicdata.elections import OCDElectionProxy
        scraped_id = getattr(self, 'scraped_id', None)
        # Try getting the OCD election via the proxy's get method
        try:
            ocd_election = self.get_ocd_election()
        except OCDElectionProxy.DoesNotExist:
            # or create a new one
            ocd_election = OCDElectionProxy.objects.create_from_calaccess(
                self.ocd_name,
                self.date,
                election_id=scraped_id,
                election_type=self.election_type,
            )
            created = True
        else:
            created = False
            # If getting an existing election, add the election_type
            ocd_election.add_election_type(self.election_type)
            # and scraped_id
            if scraped_id:
                ocd_election.add_election_id(self.scraped_id)

            ocd_election.refresh_from_db()

        return ocd_election, created

    @property
    def is_primary(self):
        """
        Returns whether or now the election was a primary.
        """
        return 'PRIMARY' in self.name.upper()

    @property
    def is_general(self):
        """
        Returns whether or now the election was a general election.
        """
        return 'GENERAL' in self.name.upper()

    @property
    def is_special(self):
        """
        Returns whether or now the election was a special election.
        """
        return 'SPECIAL' in self.name.upper()

    @property
    def is_recall(self):
        """
        Returns whether or now the election was a recall.
        """
        return 'RECALL' in self.name.upper()

    @property
    def is_partisan_primary(self):
        """
        Returns whether or not this was a primary election held in the partisan era prior to 2012.
        """
        if self.is_primary:
            if self.get_ocd_election().date.year < 2012:
                return True
        return False

    @property
    def ocd_name(self):
        """
        Return the name of the election in OCD format: {YEAR} {TYPE}.
        """
        # Add contests decided on 2/5/2008 should to an elecion named...
        if self.date == date(2008, 2, 5):
            ocd_name = "2008 PRESIDENTIAL PRIMARY AND SPECIAL ELECTIONS"
        # Add contests decided on 6/3/2008 should to an elecion named...
        elif self.date == date(2008, 6, 3):
            ocd_name = "2008 PRIMARY"
        # Otherwise return the format: "{YEAR} {ELECTION_TYPE}"
        else:
            ocd_name = '{0} {1}'.format(self.date.year, self.election_type)
        return ocd_name


class ScrapedNameMixin(object):
    """
    Tools for cleaning up scraped candidate names.
    """
    @property
    def corrected_name(self):
        """
        Returns the scraped name with any corrections made.
        """
        fixes = {
            # http://www.sos.ca.gov/elections/prior-elections/statewide-election-results/primary-election-march-7-2000/certified-list-candidates/ # noqa
            'COURTRIGHT DONNA': 'COURTRIGHT, DONNA'
        }
        return fixes.get(self.name, self.name)

    @property
    def parsed_name(self):
        """
        Return a dict of formatted Person name field values.
        """
        # sort_name is undoctored name from scrape
        d = {
            'sort_name': self.name.strip()
        }

        # parse out these suffixes: JR, SR, II, III
        suffix_pattern = r'(?:^|\s)((?:[JS]R\.?)|(?:I{2,3}))(?:,|\s|$)'
        match = re.search(suffix_pattern, d['sort_name'])
        if match:
            # replace suffix with a comma
            # and replace any double commas, strip any trailing
            d['sort_name'] = d['sort_name'].replace(match.group(), ',').replace(',,', ',')
            d['sort_name'] = re.sub(r',\s?$', '', d['sort_name']).strip()

        # split once, strip and flip the sort_name to make name
        split_name = [i.strip() for i in d['sort_name'].split(',', 1)]
        name_list = list(split_name)
        name_list.reverse()
        d['name'] = ' '.join(name_list).strip()
        d['family_name'] = split_name[0]
        if len(split_name) > 1:
            d['given_name'] = split_name[1]
        if match:
            # append suffix to end of name and given_name
            suffix = ' %s' % match.group().replace(',', '').strip()
            d['name'] += suffix
            if 'given_name' in d:
                d['given_name'] += suffix

        return d

    def parse_office_name(self):
        """
        Parse string containg the name for an office.

        Expected format is "{TYPE NAME} [{DISTRICT NUMBER}]".

        Return a dict with two keys: type and district.
        """
        office_pattern = r'^(?P<type>[A-Z ]+)(?P<district>\d{2})?$'
        try:
            parsed = re.match(office_pattern, self.office_name.upper()).groupdict()
        except AttributeError:
            parsed = {'type': None, 'district': None}
        else:
            parsed['type'] = parsed['type'].strip()
            try:
                parsed['district'] = int(parsed['district'])
            except TypeError:
                pass
        return parsed



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals
from opencivicdata.core.models import Organization
from calaccess_processed.proxies import OCDProxyModelMixin
from calaccess_processed_elections.managers import OCDPartyManager


class OCDPartyProxy(Organization, OCDProxyModelMixin):
    """
    A proxy on the OCD Organization model with helper methods for interacting with political parties.
    """
    objects = OCDPartyManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    def is_unknown(self):
        """
        Returns whether or not the provided party is unknown.
        """
        return self.name == 'UNKNOWN'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom manager for the Jurisdiction model.
"""
from __future__ import unicode_literals
from calaccess_processed.managers import BulkLoadSQLManager


class OCDJurisdictionManager(BulkLoadSQLManager):
    """
    Custom helpers for the OCD Jurisdiction model.
    """
    def california(self):
        """
        Returns California State Government jurisdiction.
        """
        from calaccess_processed_elections.proxies import OCDDivisionProxy
        qs = self.get_queryset()
        division = OCDDivisionProxy.objects.california()
        return qs.get_or_create(name='California', classification='government', division=division)[0]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Submodule for all filing-related models, managers and mixins.
"""
from .base import FilingBaseModel
from .campaign import (
    CampaignContributionBase,
    CampaignExpenditureItemBase,
    CampaignExpenditureSubItemBase,
    CampaignFinanceFilingBase,
    CampaignLoanItemBase,
    CampaignLoanReceivedItemBase,
    CampaignLoanMadeItemBase,
    Form496Filing,
    Form496FilingVersion,
    Form496Part1Item,
    Form496Part1ItemVersion,
    Form496Part2Item,
    Form496Part2ItemVersion,
    Form496Part3Item,
    Form496Part3ItemVersion,
    Form497Filing,
    Form497FilingVersion,
    Form497ItemBase,
    Form497Part1ItemBase,
    Form497Part1Item,
    Form497Part1ItemVersion,
    Form497Part2ItemBase,
    Form497Part2Item,
    Form497Part2ItemVersion,
    Form460FilingBase,
    Form460Filing,
    Form460FilingVersion,
    Form460ScheduleASummaryBase,
    Form460ScheduleASummary,
    Form460ScheduleASummaryVersion,
    Form460ScheduleAItemBase,
    Form460ScheduleAItem,
    Form460ScheduleAItemVersion,
    Form460ScheduleB1ItemBase,
    Form460ScheduleB1Item,
    Form460ScheduleB1ItemVersion,
    Form460ScheduleB2ItemBase,
    Form460ScheduleB2Item,
    Form460ScheduleB2ItemVersion,
    Form460ScheduleB2ItemBaseOld,
    Form460ScheduleB2ItemOld,
    Form460ScheduleB2ItemVersionOld,
    Form460ScheduleCSummaryBase,
    Form460ScheduleCSummary,
    Form460ScheduleCSummaryVersion,
    Form460ScheduleCItemBase,
    Form460ScheduleCItem,
    Form460ScheduleCItemVersion,
    Form460ScheduleDItemBase,
    Form460ScheduleDItem,
    Form460ScheduleDItemVersion,
    Form460ScheduleESummaryBase,
    Form460ScheduleESummary,
    Form460ScheduleESummaryVersion,
    Form460ScheduleEItem,
    Form460ScheduleEItemVersion,
    Form460ScheduleESubItem,
    Form460ScheduleESubItemVersion,
    Form460ScheduleFItemBase,
    Form460ScheduleFItem,
    Form460ScheduleFItemVersion,
    Form460ScheduleGItemBase,
    Form460ScheduleGItem,
    Form460ScheduleGItemVersion,
    Form460ScheduleHItemBase,
    Form460ScheduleHItem,
    Form460ScheduleHItemVersion,
    Form460ScheduleH2ItemBaseOld,
    Form460ScheduleH2ItemOld,
    Form460ScheduleH2ItemVersionOld,
    Form460ScheduleIItemBase,
    Form460ScheduleIItem,
    Form460ScheduleIItemVersion,
    Form461Filing,
    Form461FilingVersion,
    Form461Part5Item,
    Form461Part5ItemVersion,
    Form501FilingBase,
    Form501Filing,
    Form501FilingVersion,
)

__all__ = (
    "FilingBaseModel",
    "CampaignContributionBase",
    "CampaignExpenditureItemBase",
    "CampaignExpenditureSubItemBase",
    "CampaignFinanceFilingBase",
    "CampaignLoanItemBase",
    "CampaignLoanReceivedItemBase",
    "CampaignLoanMadeItemBase",
    "Form496Filing",
    "Form496FilingVersion",
    "Form496Part1Item",
    "Form496Part1ItemVersion",
    "Form496Part2Item",
    "Form496Part2ItemVersion",
    "Form496Part3Item",
    "Form496Part3ItemVersion",
    "Form497Filing",
    "Form497FilingVersion",
    "Form497ItemBase",
    "Form497Part1ItemBase",
    "Form497Part1Item",
    "Form497Part1ItemVersion",
    "Form497Part2ItemBase",
    "Form497Part2Item",
    "Form497Part2ItemVersion",
    "Form460FilingBase",
    "Form460Filing",
    "Form460FilingVersion",
    "Form460ScheduleASummaryBase",
    "Form460ScheduleASummary",
    "Form460ScheduleASummaryVersion",
    "Form460ScheduleAItemBase",
    "Form460ScheduleAItem",
    "Form460ScheduleAItemVersion",
    "Form460ScheduleB1ItemBase",
    "Form460ScheduleB1Item",
    "Form460ScheduleB1ItemVersion",
    "Form460ScheduleB2ItemBase",
    "Form460ScheduleB2Item",
    "Form460ScheduleB2ItemVersion",
    "Form460ScheduleB2ItemBaseOld",
    "Form460ScheduleB2ItemOld",
    "Form460ScheduleB2ItemVersionOld",
    "Form460ScheduleCSummaryBase",
    "Form460ScheduleCSummary",
    "Form460ScheduleCSummaryVersion",
    "Form460ScheduleCItemBase",
    "Form460ScheduleCItem",
    "Form460ScheduleCItemVersion",
    "Form460ScheduleDItemBase",
    "Form460ScheduleDItem",
    "Form460ScheduleDItemVersion",
    "Form460ScheduleESummaryBase",
    "Form460ScheduleESummary",
    "Form460ScheduleESummaryVersion",
    "Form460ScheduleEItem",
    "Form460ScheduleEItemVersion",
    "Form460ScheduleESubItem",
    "Form460ScheduleESubItemVersion",
    "Form460ScheduleFItemBase",
    "Form460ScheduleFItem",
    "Form460ScheduleFItemVersion",
    "Form460ScheduleGItemBase",
    "Form460ScheduleGItem",
    "Form460ScheduleGItemVersion",
    "Form460ScheduleHItemBase",
    "Form460ScheduleHItem",
    "Form460ScheduleHItemVersion",
    "Form460ScheduleH2ItemBaseOld",
    "Form460ScheduleH2ItemOld",
    "Form460ScheduleH2ItemVersionOld",
    "Form460ScheduleIItemBase",
    "Form460ScheduleIItem",
    "Form460ScheduleIItemVersion",
    "Form461Filing",
    "Form461FilingVersion",
    "Form461Part5Item",
    "Form461Part5ItemVersion",
    "Form501FilingBase",
    "Form501Filing",
    "Form501FilingVersion",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignContributionBase


class Form460ScheduleIItemBase(CampaignContributionBase):
    """
    Abstract base model for items reported on Schedule I of Form 460 filings.

    On Schedule I, campaign filers are required to report miscellaneous cash
    increases during the period covered by the filing. These include any
    transaction that increases the cash position of the filer, but is not a
    monetary contribution, loan, or loan repayment.
    """
    amount = models.DecimalField(
        verbose_name='amount',
        decimal_places=2,
        max_digits=14,
        help_text="Amount of cash increase from the contributor in the period "
                  "covered by the filing (from RCPT_CD.AMOUNT)"
    )
    receipt_description = models.CharField(
        verbose_name='receipt description',
        max_length=90,
        blank=True,
        help_text="Description of the cash increase (from RCPT_CD.CTRIB_DSCR)"
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleIItem(Form460ScheduleIItemBase):
    """
    Miscellaneous cash increases to the coffers of campaign filers.

    Includes any transaction that increases the cash position of the filer, but
    is not a monetary contribution, loan, or loan repayment.

    These transactions are itemized on Schedule I of the most recent version
    of each Form 460 filing. For miscellaneous cash increases itemized on any
    version of any Form 460 filing, see Form460ScheduleIItemVersion.

    Derived from RCPT_CD records where FORM_TYPE is 'I'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_i_items',
        null=True,
        on_delete=models.SET_NULL,
        db_constraint=False,
        help_text='Foreign key referring to the Form 460 on which the '
                  'miscellaneous cash increase was report (from RCPT_CD.'
                  'FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule I item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleIItemVersion(Form460ScheduleIItemBase):
    """
    Every version of each miscellaneous cash increase for a campaign filer.

    Includes any transaction that increases the cash position of the filer, but
    is not a monetary contribution, loan, or loan repayment.

    For miscellaneous cash increases itemized on Schedule I of the most recent
    version of each Form 460 filing, see Form460ScheduleIItem.

    Derived from RCPT_CD records where FORM_TYPE is 'I'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_i_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the miscellaneous cash increase'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule I item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom managers for models from the calaccess_scraped app.
"""
from __future__ import unicode_literals
from calaccess_processed.managers import BulkLoadSQLManager


class ScrapedIncumbentElectionManager(BulkLoadSQLManager):
    """
    Manager with custom queryset and methods on the ScrapedIncumbentElectionProxy model.
    """
    def get_queryset(self):
        """
        Override the default manager to exclude blacklisted elections.
        """
        qs = super(ScrapedIncumbentElectionManager, self).get_queryset()
        # http://www.sos.ca.gov/elections/prior-elections/special-elections/
        blacklisted_elections = (
            '2017-10-2',
            '2015-11-30',
            '2015-9-28',
            '2014-9-29',
            '2014-3-17',
            '2001-3-6'
        )
        return qs.exclude(date__in=blacklisted_elections)


class ScrapedBallotMeasureManager(BulkLoadSQLManager):
    """
    Custom manager that filters ScrapedProposition model to ballot measures.
    """
    def get_queryset(self):
        """
        Filter to ballot measures.
        """
        return super(
            ScrapedBallotMeasureManager, self
        ).get_queryset().exclude(name__icontains='RECALL')


class ScrapedRecallMeasureManager(BulkLoadSQLManager):
    """
    Custom manager that filters ScrapedProposition model to recall measures.
    """
    def get_queryset(self):
        """
        Filter to recall measures.
        """
        return super(
            ScrapedRecallMeasureManager, self
        ).get_queryset().filter(name__icontains='RECALL')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom manager for OCD organization models.
"""
from __future__ import unicode_literals
from django.db.models import Count
from calaccess_processed.managers import BulkLoadSQLManager

# Logging
import logging
logger = logging.getLogger(__name__)


class OCDOrganizationManager(BulkLoadSQLManager):
    """
    Custom helpers for the OCD Organization model.
    """
    def senate(self):
        """
        Returns state senate organization.
        """
        return self.get_queryset().get_or_create(
            name='California State Senate',
            classification='upper',
        )[0]

    def assembly(self):
        """
        Returns state assembly organization.
        """
        return self.get_queryset().get_or_create(
            name='California State Assembly',
            classification='lower',
        )[0]

    def executive_branch(self):
        """
        Returns executive branch organization.
        """
        return self.get_queryset().get_or_create(
            name='California State Executive Branch',
            classification='executive',
        )[0]

    def secretary_of_state(self):
        """
        Returns secretary of state organization.
        """
        return self.get_queryset().get_or_create(
            name='California Secretary of State',
            classification='executive',
            parent=self.executive_branch(),
        )[0]

    def elections_division(self):
        """
        Returns the elections division of the secretary of state organization.
        """
        return self.get_queryset().get_or_create(
            name='Elections Division',
            classification='executive',
            parent=self.secretary_of_state(),
        )[0]

    def board_of_equalization(self):
        """
        Returns board of equalization organization.
        """
        return self.get_queryset().get_or_create(
            name='State Board of Equalization',
            parent=self.executive_branch(),
        )[0]


class OCDMembershipManager(BulkLoadSQLManager):
    """
    Manager for custom methods on the OCDMembershipProxy model.
    """
    def get_or_create_from_calaccess(self, incumbent):
        """
        Get or create and OCD Membership from a scraped incumbent.
        """
        from calaccess_processed_elections.proxies import OCDPersonProxy, OCDPostProxy

        # Get or create post
        post, post_created = OCDPostProxy.objects.get_or_create_by_name(incumbent.office_name)
        if post_created:
            logger.debug(' Created new Post: %s' % post.label)

        # Get or create person
        person, person_created = OCDPersonProxy.objects.get_or_create_from_calaccess(
            incumbent.parsed_name,
            candidate_filer_id=incumbent.scraped_id,
        )
        if person_created:
            logger.debug(' Created new Person: %s' % person.name)

        # Get or create membership for post and person
        membership, membership_created = self.get_queryset().get_or_create(
            person=person,
            post=post,
            role=post.role,
            organization=post.organization,
        )

        incumbent_name = incumbent.parsed_name['name']
        if membership.person_name != incumbent_name:
            membership.person_name == incumbent_name
            membership.save()
        membership.person_proxy.add_other_name(
            incumbent_name, 'From scraped incumbent record'
        )

        return membership, membership_created

    def get_duplicates(self):
        """
        Return a QuerySet with duplicate Membership instances.
        """
        # group memberships by post_id and person_id
        # check if any group has more than one row.
        return self.get_queryset().values(
            'post', 'person'
        ).annotate(row_count=Count('id')).filter(row_count__gt=1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the models from submodules and thread them together.
"""
from .ballotmeasurecontests import OCDFlatBallotMeasureContestProxy
from .candidacies import OCDFlatCandidacyProxy
from .retentioncontests import OCDFlatRetentionContestProxy


__all__ = (
    "OCDFlatBallotMeasureContestProxy",
    "OCDFlatCandidacyProxy",
    "OCDFlatRetentionContestProxy"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom managers for the Division model.
"""
from __future__ import unicode_literals
from calaccess_processed.managers import BulkLoadSQLManager


class OCDAssemblyDivisionManager(BulkLoadSQLManager):
    """
    Custom manager for state assembly OCD Divisions.
    """
    def get_queryset(self):
        """
        Filters down to state assembly divisions.
        """
        qs = super(OCDAssemblyDivisionManager, self).get_queryset()
        return qs.filter(subid1='ca', subtype2='sldl')


class OCDSenateDivisionManager(BulkLoadSQLManager):
    """
    Custom manager for state senate OCD Divisions.
    """
    def get_queryset(self):
        """
        Filters down to state senate divisions.
        """
        qs = super(OCDSenateDivisionManager, self).get_queryset()
        return qs.filter(subid1='ca', subtype2='sldu')


class OCDCaliforniaDivisionManager(BulkLoadSQLManager):
    """
    Custom manager for OCD Divisions in California.
    """
    def get_queryset(self):
        """
        Filters down to divisions in California.
        """
        qs = super(OCDCaliforniaDivisionManager, self).get_queryset()
        return qs.filter(subid1='ca')

    def california(self):
        """
        Returns state of California division.
        """
        qs = super(OCDCaliforniaDivisionManager, self).get_queryset()
        return qs.get(id='ocd-division/country:us/state:ca')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Admins for Schedule 497, the late contribution report.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_processed_filings import models
from calaccess_raw.admin.base import BaseAdmin


@admin.register(models.Form497Filing)
class Form497FilingAdmin(BaseAdmin):
    """
    Custom admin for the Form497Filing model.
    """
    list_display = (
        'filing_id',
        'amendment_count',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
    )


@admin.register(models.Form497FilingVersion)
class Form497FilingVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form497FilingVersion model.
    """
    list_display = (
        'filing',
        'amend_id',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
    )


@admin.register(models.Form497Part1Item)
class Form497Part1ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form497Part1Item model.
    """
    list_display = (
        'filing',
        'line_item',
        'date_received',
        'amount_received',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form497Part1ItemVersion)
class Form497Part1ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form497Part1ItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'date_received',
        'amount_received',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form497Part2Item)
class Form497Part2ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form497Part2Item model.
    """
    list_display = (
        'filing',
        'line_item',
        'date_received',
        'amount_received',
        'transaction_id',
        'recipient_code',
        'recipient_lastname',
        'recipient_firstname',
    )


@admin.register(models.Form497Part2ItemVersion)
class Form497Part2ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form497Part2ItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'date_received',
        'amount_received',
        'transaction_id',
        'recipient_code',
        'recipient_lastname',
        'recipient_firstname',
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for OCD Filing related models..
"""
from __future__ import unicode_literals

# Models
from calaccess_processed.proxies import OCDProxyModelMixin
from opencivicdata.campaign_finance.models import Transaction, TransactionIdentifier

# Managers
from calaccess_processed_campaignfinance.managers import OCDTransactionManager, CampaignFinanceManager

# Logging
import logging
logger = logging.getLogger(__name__)


class OCDTransactionProxy(Transaction, OCDProxyModelMixin):
    """
    A proxy on the OCD Transaction model.
    """
    objects = OCDTransactionManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDTransactionIdentifierProxy(TransactionIdentifier, OCDProxyModelMixin):
    """
    A proxy on the OCD TransactionIdentifier model.
    """
    objects = CampaignFinanceManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the admins from submodules and thread them together.
"""
from .a import (
    Form460ScheduleASummaryBase,
    Form460ScheduleASummary,
    Form460ScheduleASummaryVersion,
    Form460ScheduleAItemBase,
    Form460ScheduleAItem,
    Form460ScheduleAItemVersion
)
from .b import (
    Form460ScheduleB1ItemBase,
    Form460ScheduleB1Item,
    Form460ScheduleB1ItemVersion,
    Form460ScheduleB2ItemBase,
    Form460ScheduleB2Item,
    Form460ScheduleB2ItemVersion,
    Form460ScheduleB2ItemBaseOld,
    Form460ScheduleB2ItemOld,
    Form460ScheduleB2ItemVersionOld
)
from .c import (
    Form460ScheduleCSummaryBase,
    Form460ScheduleCSummary,
    Form460ScheduleCSummaryVersion,
    Form460ScheduleCItemBase,
    Form460ScheduleCItem,
    Form460ScheduleCItemVersion
)
from .d import (
    Form460ScheduleDItemBase,
    Form460ScheduleDItem,
    Form460ScheduleDItemVersion
)
from .e import (
    Form460ScheduleESummaryBase,
    Form460ScheduleESummary,
    Form460ScheduleESummaryVersion,
    Form460ScheduleEItem,
    Form460ScheduleEItemVersion,
    Form460ScheduleESubItem,
    Form460ScheduleESubItemVersion
)
from .f import (
    Form460ScheduleFItemBase,
    Form460ScheduleFItem,
    Form460ScheduleFItemVersion
)
from .g import (
    Form460ScheduleGItemBase,
    Form460ScheduleGItem,
    Form460ScheduleGItemVersion
)
from .h import (
    Form460ScheduleHItemBase,
    Form460ScheduleHItem,
    Form460ScheduleHItemVersion,
    Form460ScheduleH2ItemBaseOld,
    Form460ScheduleH2ItemOld,
    Form460ScheduleH2ItemVersionOld
)
from .i import (
    Form460ScheduleIItemBase,
    Form460ScheduleIItem,
    Form460ScheduleIItemVersion
)


__all__ = (
    "Form460ScheduleASummaryBase",
    "Form460ScheduleASummary",
    "Form460ScheduleASummaryVersion",
    "Form460ScheduleAItemBase",
    "Form460ScheduleAItem",
    "Form460ScheduleAItemVersion",
    "Form460ScheduleB1ItemBase",
    "Form460ScheduleB1Item",
    "Form460ScheduleB1ItemVersion",
    "Form460ScheduleB2ItemBase",
    "Form460ScheduleB2Item",
    "Form460ScheduleB2ItemVersion",
    "Form460ScheduleB2ItemBaseOld",
    "Form460ScheduleB2ItemOld",
    "Form460ScheduleB2ItemVersionOld",
    "Form460ScheduleCSummaryBase",
    "Form460ScheduleCSummary",
    "Form460ScheduleCSummaryVersion",
    "Form460ScheduleCItemBase",
    "Form460ScheduleCItem",
    "Form460ScheduleCItemVersion",
    "Form460ScheduleDItemBase",
    "Form460ScheduleDItem",
    "Form460ScheduleDItemVersion",
    "Form460ScheduleESummaryBase",
    "Form460ScheduleESummary",
    "Form460ScheduleESummaryVersion",
    "Form460ScheduleEItem",
    "Form460ScheduleEItemVersion",
    "Form460ScheduleESubItem",
    "Form460ScheduleESubItemVersion",
    "Form460ScheduleFItemBase",
    "Form460ScheduleFItem",
    "Form460ScheduleFItemVersion",
    "Form460ScheduleGItemBase",
    "Form460ScheduleGItem",
    "Form460ScheduleGItemVersion",
    "Form460ScheduleHItemBase",
    "Form460ScheduleHItem",
    "Form460ScheduleHItemVersion",
    "Form460ScheduleH2ItemBaseOld",
    "Form460ScheduleH2ItemOld",
    "Form460ScheduleH2ItemVersionOld",
    "Form460ScheduleIItemBase",
    "Form460ScheduleIItem",
    "Form460ScheduleIItemVersion"
)



# Generated by Django 2.0.4 on 2018-04-25 00:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0003_auto_20180424_2300'),
    ]

    operations = [
        migrations.CreateModel(
            name='Form496Part1Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('candidate_title', models.CharField(blank=True, help_text='Name title of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMT)', max_length=10, verbose_name='candidate title')),
                ('candidate_lastname', models.CharField(blank=True, help_text='Last name of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAML)', max_length=200, verbose_name='candidate lastname')),
                ('candidate_firstname', models.CharField(help_text='First name of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMF)', max_length=45, verbose_name='candidate firstname')),
                ('candidate_name_suffix', models.CharField(blank=True, help_text='Name suffix of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMS)', max_length=10, verbose_name='candidate name suffix')),
                ('candidate_office_code', models.CharField(blank=True, help_text='Code describing the office sought sought by the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.OFFICE_CD)', max_length=3, verbose_name='candidate office code')),
                ('ballot_measure_name', models.CharField(blank=True, help_text='Name of the ballot measure supported or opposed by the recipient (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_NAME)', max_length=200, verbose_name='Ballot measure name')),
                ('ballot_measure_number', models.CharField(blank=True, help_text='Ballot measure number (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_NUM)', max_length=7, verbose_name='Ballot measure number')),
                ('ballot_measure_jurisdiction', models.CharField(blank=True, help_text='Jurisdiction of the ballot measure supported or opposed by the recipient (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_JURIS)', max_length=40, verbose_name='ballot measure jurisdiction')),
                ('support_opposition_code', models.CharField(blank=True, choices=[('O', 'Opposition'), ('S', 'Support')], help_text='Code describing whether the contribuitor supports or opposesthe candidate or ballot measure (from CVR_CAMPAIGN_DISCLOSURE_CD.SUP_OPP_CD)', max_length=1, verbose_name='support opposition code')),
                ('filing', models.ForeignKey(help_text='Unique identification number for the Schedule 496 filing (from S496_CD.FILING_ID)', null=True, on_delete=django.db.models.deletion.SET_NULL, to='calaccess_processed_filings.Form496Filing')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) Part 1 item',
            },
        ),
        migrations.CreateModel(
            name='Form496Part1ItemVersion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('candidate_title', models.CharField(blank=True, help_text='Name title of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMT)', max_length=10, verbose_name='candidate title')),
                ('candidate_lastname', models.CharField(blank=True, help_text='Last name of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAML)', max_length=200, verbose_name='candidate lastname')),
                ('candidate_firstname', models.CharField(help_text='First name of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMF)', max_length=45, verbose_name='candidate firstname')),
                ('candidate_name_suffix', models.CharField(blank=True, help_text='Name suffix of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMS)', max_length=10, verbose_name='candidate name suffix')),
                ('candidate_office_code', models.CharField(blank=True, help_text='Code describing the office sought sought by the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.OFFICE_CD)', max_length=3, verbose_name='candidate office code')),
                ('ballot_measure_name', models.CharField(blank=True, help_text='Name of the ballot measure supported or opposed by the recipient (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_NAME)', max_length=200, verbose_name='Ballot measure name')),
                ('ballot_measure_number', models.CharField(blank=True, help_text='Ballot measure number (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_NUM)', max_length=7, verbose_name='Ballot measure number')),
                ('ballot_measure_jurisdiction', models.CharField(blank=True, help_text='Jurisdiction of the ballot measure supported or opposed by the recipient (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_JURIS)', max_length=40, verbose_name='ballot measure jurisdiction')),
                ('support_opposition_code', models.CharField(blank=True, choices=[('O', 'Opposition'), ('S', 'Support')], help_text='Code describing whether the contribuitor supports or opposesthe candidate or ballot measure (from CVR_CAMPAIGN_DISCLOSURE_CD.SUP_OPP_CD)', max_length=1, verbose_name='support opposition code')),
                ('filing_version', models.ForeignKey(help_text='Foreign key referring to the version of the Schedule 496 that includes the received contribution', null=True, on_delete=django.db.models.deletion.SET_NULL, to='calaccess_processed_filings.Form496FilingVersion')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) Part 1 item version',
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utilities for correcting connection between candidates and parties.
"""
import os
import csv
from django.apps import apps


def candidate_party(candidate_name, year, election_type, office):
    """
    Returns the correct OCD party organization object for a given candidate name, year, election_type and office.

    Returns None if no correction is found.
    """
    from calaccess_processed_elections.proxies import OCDPartyProxy

    # Get the path to our corrections file
    app = apps.get_app_config("calaccess_processed_elections")
    module_dir = os.path.abspath(os.path.dirname(app.module.__file__))
    corrections_path = os.path.join(module_dir, 'corrections', "candidate_party.csv")

    # Open up the corrections
    with open(corrections_path, 'r') as f:
        corrections = csv.DictReader(f)
        # Filter down to the ones we've corrected
        corrections = [d for d in corrections if d['party']]

    # Filter down to the ones that match
    matches = [
        d['party'] for d in corrections if (
            d['candidate_name'] == candidate_name
            and str(d['year']) == str(year)
            and d['election_type'] == election_type
            and d['office'] == office
        )
    ]

    # If there's more than one result throw an error
    if len(matches) > 1:
        raise Exception('More than one correction found.')
    # If there's no match return None
    elif len(matches) == 0:
        return None
    # If there's only one match return that
    else:
        return OCDPartyProxy.objects.get_by_name(matches[0])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for tracking models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_processed import models
from calaccess_raw.admin.base import BaseAdmin


class ProcessedDataZipInline(admin.TabularInline):
    """
    Custom inline admin for the ProcessedDataZip model.
    """
    model = models.ProcessedDataZip
    readonly_fields = ('zip_archive', 'created_datetime', 'pretty_zip_size')
    fields = readonly_fields

    def has_add_permission(self, request, obj=None):
        """
        Disable adding of zips.
        """
        return False


@admin.register(models.ProcessedDataVersion)
class ProcessedDataVersionAdmin(BaseAdmin):
    """
    Custom admin for the ProcessedDataVersion model.
    """
    list_display = (
        "id",
        "raw_version",
        "process_start_datetime",
        "process_finish_datetime",
    )
    list_display_links = ('process_start_datetime',)
    list_filter = ("process_finish_datetime",)

    inlines = [
        ProcessedDataZipInline,
    ]


@admin.register(models.ProcessedDataFile)
class ProcessedDataFileAdmin(BaseAdmin):
    """
    Custom admin for the ProcessedDataFile model.
    """
    list_display = (
        "id",
        "version",
        "file_name",
        "records_count",
    )
    list_display_links = ('id', 'file_name',)
    list_filter = ("version__process_start_datetime",)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Basic configuration for the application.
"""
from __future__ import unicode_literals
import os
from django.apps import AppConfig


class CalAccessProcessedFilingsConfig(AppConfig):
    """
    Application configuration.
    """
    name = 'calaccess_processed_filings'
    verbose_name = "CAL-ACCESS processed data: Filings"
    default_auto_field = 'django.db.models.AutoField'

    # Where SQL files are stored in this application
    sql_directory_path = os.path.join(os.path.dirname(__file__), 'sql')

    def get_filing_model_lookup(self):
        """
        Returns a dictionary with the names mapped to models.
        """
        return dict((m.__name__, m) for m in self.get_filing_models())

    def get_filing_models(self):
        """
        Returns models from the "filings" group that mirror the structure of CAL-ACCESS forms.
        """
        # Get all the models for this app
        model_list = self.get_models()

        # Filter out any abstract ones
        model_list = [m for m in model_list if not m._meta.abstract]

        # Filter out any proxies
        model_list = [m for m in model_list if not m._meta.proxy]

        # Filter down to just models with "filing" in the name
        model_list = [m for m in model_list if 'filings' in str(m)]

        # Return what's left
        return model_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Abstract base models for campaign finance-related filings and transactions.
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel


class CampaignExpenditureItemBase(FilingBaseModel):
    """
    Abstract base model for payments made by or on behalf of campaign filers.

    These transactions are itemized on Schedules D, E and G of Form 460 filings
    and stored in the EXPN_CD table.
    """
    line_item = models.IntegerField(
        verbose_name='line item',
        help_text="Line number of the filing form where the payment is "
                  "itemized (from EXPN_CD.LINE_ITEM)",
    )
    PAYEE_CODE_CHOICES = (
        ('BNM', "Ballot measure's name/title"),
        ('CAO', "Candidate/officeholder"),
        ('COM', "Committee"),
        ('IND', "Individual"),
        ('MBR', "Member of Associaton"),
        ('OFF', "Officer"),
        ('OTH', "Other"),
        ('PTY', "Political Party"),
        ('RCP', "Recipient committee"),
        ('SCC', "Small Contributor Committee"),
        ('???', "Unknown value"),
    )
    payee_code = models.CharField(
        verbose_name='payee code',
        max_length=3,
        blank=True,
        choices=PAYEE_CODE_CHOICES,
        help_text='Code describing the payee (from EXPN_CD.ENTITY_CD)',
    )
    payee_committee_id = models.CharField(
        verbose_name='payee committee id',
        max_length=9,
        blank=True,
        help_text="Payee's filer identification number, if it is a "
                  "committee (from EXPN_CD.CMTE_ID)",
    )
    payee_title = models.CharField(
        verbose_name='payee title',
        max_length=10,
        blank=True,
        help_text='Name title of the payee (from EXPN_CD.PAYEE_NAMT)',
    )
    payee_lastname = models.CharField(
        verbose_name='payee lastname',
        max_length=200,
        blank=True,
        help_text='Last name of the payee or business name (from '
                  'EXPN_CD.PAYEE_NAML)',
    )
    payee_firstname = models.CharField(
        verbose_name='payee firstname',
        max_length=45,
        help_text='First name of the payee (from EXPN_CD.PAYEE_NAMF)',
    )
    payee_name_suffix = models.CharField(
        verbose_name='payee name suffix',
        max_length=10,
        blank=True,
        help_text='Name suffix of the payee (from EXPN_CD.PAYEE_NAMS)',
    )
    payee_city = models.CharField(
        verbose_name='payee city',
        max_length=30,
        blank=True,
        help_text='City of the payee (from EXPN_CD.PAYEE_CITY)',
    )
    payee_state = models.CharField(
        verbose_name='payee state',
        max_length=2,
        blank=True,
        help_text='State of the payee (from EXPN_CD.PAYEE_ST)',
    )
    payee_zip = models.CharField(
        verbose_name='payee zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'payee (from EXPN_CD.PAYEE_ZIP4)',
    )
    treasurer_title = models.CharField(
        verbose_name='treasurer title',
        max_length=10,
        blank=True,
        help_text="Name title of the payee committee's treasurer (from "
                  "EXPN_CD.TRES_NAMT)",
    )
    treasurer_lastname = models.CharField(
        verbose_name='treasurer lastname',
        max_length=200,
        blank=True,
        help_text="Last name of the payee committee's treasurer (from "
                  "EXPN_CD.TRES_NAML)",
    )
    treasurer_firstname = models.CharField(
        verbose_name='treasurer firstname',
        max_length=45,
        help_text="First name of the payee committee's treasurer (from "
                  "EXPN_CD.TRES_NAMF)",
    )
    treasurer_name_suffix = models.CharField(
        verbose_name='treasurer name suffix',
        max_length=10,
        blank=True,
        help_text="Name suffix of the payee committee's treasurer (from "
                  "EXPN_CD.TRES_NAMS)",
    )
    treasurer_city = models.CharField(
        verbose_name='treasurer city',
        max_length=30,
        blank=True,
        help_text="City of the payee committee's treasurer (from EXPN_CD."
                  "TRES_CITY)",
    )
    treasurer_state = models.CharField(
        verbose_name='treasurer state',
        max_length=2,
        blank=True,
        help_text="State of the payee committee's treasurer (from EXPN_CD."
                  "TRES_ST)",
    )
    treasurer_zip = models.CharField(
        verbose_name='treasurer zip',
        max_length=10,
        blank=True,
        help_text="Zip code (usually zip5, sometimes zip9) of the payee "
                  "committee's treasurer (from EXPN_CD.TRES_ZIP4)",
    )
    PAYMENT_CODE_CHOICES = (
        ('CMP', 'Campaign paraphernalia/miscellaneous'),
        ('CNS', 'Campaign consultants'),
        ('CTB', 'Contribution (if nonmonetary, explain)*'),
        ('CVC', 'Civic donations'),
        ('FIL', 'Candidate filing/ballot feeds'),
        ('FND', 'Fundraising events'),
        ('IKD', 'In-kind contribution (nonmonetary)'),
        ('IND', 'Independent expenditure supporting/opposing others (explain)*'),
        ('LEG', 'Legal defense'),
        ('LIT', 'Campaign literature and mailings'),
        ('LON', 'Loan'),
        ('MBR', 'Member communications'),
        ('MON', 'Monetary contribution'),
        ('MTG', 'Meetings and appearances'),
        ('OFC', 'Office expenses'),
        ('PET', 'Petition circulating'),
        ('PHO', 'Phone banks'),
        ('POL', 'Polling and survey research'),
        ('POS', 'Postage, delivery and messenger services'),
        ('PRO', 'Professional services (legal, accounting)'),
        ('PRT', 'Print ads'),
        ('RAD', 'Radio airtime and production costs'),
        ('RFD', 'Returned contributions'),
        ('SAL', 'Campaign workers salaries'),
        ('TEL', 'T.V. or cable airtime and production costs'),
        ('TRC', 'Candidate travel, lodging and meals (explain)'),
        ('TRS', 'Staff/spouse travel, lodging and meals (explain)'),
        ('TSF', 'Transfer between committees of the same candidate/sponsor'),
        ('VOT', 'Voter registration'),
        ('WEB', 'Information technology costs (internet, e-mail)'),
        ('???', "Unknown value"),
    )
    payment_code = models.CharField(
        verbose_name='payment code',
        max_length=3,
        blank=True,
        choices=PAYMENT_CODE_CHOICES,
        help_text='Code describing the payment (from EXPN_CD.EXPN_CODE)',
    )
    payment_description = models.CharField(
        verbose_name="payment description",
        max_length=400,
        blank=True,
        help_text="Purpose of payment and/or description/explanation (from "
                  "EXPN_CD.EXPN_DSCR)",
    )
    amount = models.DecimalField(
        verbose_name='amount',
        decimal_places=2,
        max_digits=14,
        help_text="Amount paid to the payee in the period covered by the "
                  "filing (from EXPN_CD.AMOUNT)",
    )
    cumulative_ytd_amount = models.DecimalField(
        decimal_places=2,
        max_digits=14,
        null=True,
        help_text="Cumulative year-to-date amount given or spent by the filer "
                  "in support or opposition of the candidate or ballot "
                  "measure as of the Form 460's filing date (from EXPN_CD."
                  "CUM_YTD)",
    )
    expense_date = models.DateField(
        verbose_name="expense date",
        null=True,
        help_text="Date or expense (from EXPN_CD.EXPN_DATE)",
    )
    check_number = models.CharField(
        verbose_name='expense check number',
        max_length=20,
        blank=True,
        help_text="Optional check number for the payment made by the campaign "
                  "filer (from EXPN_CD.EXPN_CHKNO)",
    )
    SUPPORT_OPPOSE_CHOICES = (
        ('S', 'Support'),
        ('O', 'Oppose'),
        ('?', 'Unknown value')
    )
    support_oppose_code = models.CharField(
        verbose_name='support oppose code',
        max_length=1,
        blank=True,
        choices=SUPPORT_OPPOSE_CHOICES,
        help_text='If applicable, code indicating whether the payment went '
                  'toward supporting or opposing a candidate/ballot measure '
                  '(from EXPN_CD.SUP_OPP_CD)',
    )
    ballot_measure_jurisdiction = models.CharField(
        verbose_name='ballot measure jurisdiction',
        max_length=40,
        blank=True,
        help_text="If the payment went toward supporting/opposing a ballot "
                  "measure, the jurisdiction subject to the ballot measure "
                  "(from EXPN_CD.BAL_JURIS)",
    )
    ballot_measure_name = models.CharField(
        verbose_name='ballot measure name',
        max_length=200,
        blank=True,
        help_text="If the payment went toward supporting/opposing a ballot "
                  "measure, name of the ballot measure (from EXPN_CD.BAL_NAME "
                  " or EXPN_CD.CAND_NAML)"
    )
    ballot_measure_num = models.CharField(
        verbose_name='ballot measure number',
        max_length=7,
        blank=True,
        help_text="If the payment went toward supporting/opposing a ballot "
                  "measure, ballot number or letter (from EXPN_CD.BAL_NUM)",
    )
    candidate_title = models.CharField(
        verbose_name='candidate title',
        max_length=10,
        blank=True,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  " name title of the candidate (from EXPN_CD.CAND_NAMT)",
    )
    candidate_lastname = models.CharField(
        verbose_name='candidate lastname',
        max_length=200,
        blank=True,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  " last name of the candidate or business name (from EXPN_CD."
                  "CAND_NAML)",
    )
    candidate_firstname = models.CharField(
        verbose_name='candidate firstname',
        max_length=45,
        blank=True,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  " first name of the candidate (from EXPN_CD.CAND_NAMF)",
    )
    candidate_name_suffix = models.CharField(
        verbose_name='candidate name suffix',
        max_length=10,
        blank=True,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  " name suffix of the candidate (from EXPN_CD.CAND_NAMS)",
    )
    JURISDICTION_CODE_CHOICES = (
        ('ASM', 'Assembly District'),
        ('BOE', 'Board of Equalization District'),
        ('CIT', 'City'),
        ('CTY', 'County'),
        ('LOC', 'Local'),
        ('OTH', 'Other'),
        ('SEN', 'Senate District'),
        ('STW', 'Statewide'),
        ('???', 'Statewide'),
    )
    candidate_jurisdiction_code = models.CharField(
        verbose_name='candidate jurisdiction',
        max_length=3,
        blank=True,
        choices=JURISDICTION_CODE_CHOICES,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  "code indicating the jurisdiction of the office (from"
                  " EXPN_CD.JURIS_CD)",
    )
    candidate_jurisdiction_description = models.CharField(
        verbose_name='candidate jurisdiciton description',
        max_length=40,
        blank=True,
        help_text="If the payment went toward supporting/opposing a county, "
                  "city or local candidate, full description of the office "
                  "(from EXPN_CD.JURIS_DSCR)",
    )
    candidate_district = models.CharField(
        verbose_name='candidate district',
        max_length=3,
        blank=True,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  "for state senate, assembly or local board of education, the"
                  "district of the office (from EXPN_CD.DIST_NO)",
    )
    OFFICE_SOUGHT_HELD_CHOICES = (
        ('S', 'SOUGHT'),
        ('H', 'HELD'),
    )
    office_sought_held = models.CharField(
        verbose_name='office sought or held',
        max_length=1,
        blank=True,
        choices=OFFICE_SOUGHT_HELD_CHOICES,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  "code indicating if the candidate is seeking or currently "
                  "holds the office (from EXPN_CD.OFF_S_H_CD)",
    )
    OFFICE_CODE_CHOICES = (
        ('APP', "State Appellate Court Justice"),
        ('ASM', 'State Assembly Person'),
        ('ASR', 'Assessor'),
        ('ATT', 'Attorney General'),
        ('BED', 'Board of Education'),
        ('BOE', 'Board of Equalization Member'),
        ('BSU', 'Board of Supervisors'),
        ('CAT', 'City Attorney'),
        ('CCB', 'Community College Board'),
        ('CCM', 'City Council Member'),
        ('CON', 'State Controller'),
        ('COU', 'County Counsel'),
        ('CSU', 'County Supervisor'),
        ('CTR', 'Local Controller'),
        ('DAT', 'District Attorney'),
        ('GOV', 'Governor'),
        ('INS', 'Insurance Commissioner'),
        ('LTG', 'Lieutenant Governor'),
        ('MAY', 'Mayor'),
        ('OTH', 'Other'),
        ('PDR', 'Public Defender'),
        ('PER', 'Public Employees Retirement System'),
        ('PLN', 'Planning Commissioner'),
        ('SCJ', 'Superior Court Judge'),
        ('SEN', 'State Senator'),
        ('SHC', 'Sheriff-Coroner'),
        ('SOS', 'Secretary of State'),
        ('SPM', 'Supreme Court Justice'),
        ('SUP', 'Superintendent of Public Instruction'),
        ('TRE', 'State Treasurer'),
        ('TRS', 'Local Treasurer'),
        ('???', 'Unknown value'),
    )
    office_code = models.CharField(
        verbose_name='office code',
        max_length=3,
        blank=True,
        choices=OFFICE_CODE_CHOICES,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  "code describing the office (from EXPN_CD.OFFICE_CD)",
    )
    office_description = models.CharField(
        verbose_name='office description',
        max_length=40,
        blank=True,
        help_text="If the payment went toward supporting/opposing a candidate,"
                  "description of the office (from EXPN_CD.OFFIC_DSCR)",
    )
    transaction_id = models.CharField(
        verbose_name='transaction id',
        max_length=20,
        help_text='Identifies a unique transaction across versions of the a '
                  'given Form 460 filing (from EXPN_CD.TRAN_ID)',
    )
    memo_reference_number = models.CharField(
        verbose_name='memo reference number',
        max_length=20,
        blank=True,
        help_text="A value assigned by the filer which refers to the item's"
                  "footnote in the TEXT_MEMO_CD table (from "
                  "EXPN_CD.MEMO_REFNO)",
    )

    class Meta:
        """
        Model options.
        """
        abstract = True


class CampaignExpenditureSubItemBase(CampaignExpenditureItemBase):
    """
    Abstract base model for sub-items of campaign expenditures.

    A sub-item is a transaction where the amount is lumped into another
    "parent" payment reported elsewhere on the filing.
    """
    parent_transaction_id = models.CharField(
        verbose_name='parent transaction id',
        max_length=20,
        blank=True,
        help_text='Refers to a parent transaction itemized on the same Form '
                  '460 filing (from EXPN_CD.BAKREF_TID)',
    )

    class Meta:
        """
        Model options.
        """
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load OCD elections models with data extracted and scraped from CAL-ACCESS.
"""
import os
from django.apps import apps
from django.conf import settings
from django.utils.timezone import now
from django.core.management import call_command
from calaccess_processed.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Archive flat files of CAL-ACCESS data.
    """
    help = 'Archive flat files of CAL-ACCESS data'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # archive if django project setting enabled
        if not getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            self.log("Archiving is deactivated. Check your CALACCESS_STORE_ARCHIVE setting.")
            return

        # then archive
        if self.verbosity > 2:
            self.log(' Archiving OCD processed data files.')

        # Get the tracking model instance for this version
        self.processed_version = self.get_or_create_processed_version()[0]

        # create subdirectory in processed_data_dir, if missing
        filings_data_path = os.path.join(self.processed_data_dir, 'flat')
        os.path.isdir(filings_data_path) or os.makedirs(filings_data_path)

        # now do flat files
        flat_file_list = apps.get_app_config("calaccess_processed_flatfiles").get_flat_names_list()
        for f in flat_file_list:
            processed_data_file, created = self.processed_version.files.get_or_create(file_name=f)
            processed_data_file.process_start_datetime = now()
            processed_data_file.save()

            call_command('archivecalaccessflatfile', f)
            processed_data_file.refresh_from_db()
            processed_data_file.process_finish_datetime = now()
            processed_data_file.save()

        # Wrap it up
        self.success('Done!')



# Generated by Django 2.0.4 on 2018-04-24 23:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0002_auto_20180424_2226'),
    ]

    operations = [
        migrations.CreateModel(
            name='Form496Part2Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('line_item', models.IntegerField(db_index=True, help_text='Line number of the filing form where the transaction is itemized (from S496_CD.LINE_ITEM)', verbose_name='line item')),
                ('expense_date', models.DateField(help_text='Date or expense (from S496_CD.EXP_DATE)', null=True, verbose_name='expense date')),
                ('amount', models.DecimalField(decimal_places=2, help_text='Amount paid to the payee in the period covered by the filing (from S496_CD.AMOUNT)', max_digits=14, verbose_name='amount')),
                ('transaction_id', models.CharField(db_index=True, help_text='Identifies a unique transaction across versions of the a given Schedule 496 filing (from S496_CD.TRAN_ID)', max_length=20, verbose_name='transaction id')),
                ('payment_description', models.CharField(blank=True, help_text='Purpose of payment and/or description/explanation (from S496_CD.EXPN_DSCR)', max_length=400, verbose_name='payment description')),
                ('memo_code', models.CharField(blank=True, help_text='A description offered by the filer', max_length=500, verbose_name='memo code')),
                ('memo_reference_number', models.CharField(blank=True, help_text='Reference number for the memo attached to the transaction (from S496_CD.MEMO_REFNO)', max_length=20, verbose_name='memo reference number')),
                ('filing', models.ForeignKey(db_constraint=False, help_text='Unique identification number for the Schedule 496 filing (from S496_CD.FILING_ID)', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='independent_expenditures_made', to='calaccess_processed_filings.Form496Filing')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) Part 2 item',
            },
        ),
        migrations.CreateModel(
            name='Form496Part2ItemVersion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('line_item', models.IntegerField(db_index=True, help_text='Line number of the filing form where the transaction is itemized (from S496_CD.LINE_ITEM)', verbose_name='line item')),
                ('expense_date', models.DateField(help_text='Date or expense (from S496_CD.EXP_DATE)', null=True, verbose_name='expense date')),
                ('amount', models.DecimalField(decimal_places=2, help_text='Amount paid to the payee in the period covered by the filing (from S496_CD.AMOUNT)', max_digits=14, verbose_name='amount')),
                ('transaction_id', models.CharField(db_index=True, help_text='Identifies a unique transaction across versions of the a given Schedule 496 filing (from S496_CD.TRAN_ID)', max_length=20, verbose_name='transaction id')),
                ('payment_description', models.CharField(blank=True, help_text='Purpose of payment and/or description/explanation (from S496_CD.EXPN_DSCR)', max_length=400, verbose_name='payment description')),
                ('memo_code', models.CharField(blank=True, help_text='A description offered by the filer', max_length=500, verbose_name='memo code')),
                ('memo_reference_number', models.CharField(blank=True, help_text='Reference number for the memo attached to the transaction (from S496_CD.MEMO_REFNO)', max_length=20, verbose_name='memo reference number')),
                ('filing_version', models.ForeignKey(help_text='Foreign key referring to the version of the Schedule 496 that includes the given expenditure', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='independent_expenditures_made', to='calaccess_processed_filings.Form496FilingVersion')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) Part 2 item version',
            },
        ),
        migrations.AlterUniqueTogether(
            name='form496part2itemversion',
            unique_together={('filing_version', 'line_item')},
        ),
        migrations.AlterIndexTogether(
            name='form496part2itemversion',
            index_together={('filing_version', 'line_item')},
        ),
        migrations.AlterUniqueTogether(
            name='form496part2item',
            unique_together={('filing', 'line_item')},
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from .ballotmeasurecontests import (
    OCDBallotMeasureContestProxy,
    OCDBallotMeasureContestIdentifierProxy,
    OCDBallotMeasureContestOptionProxy,
    OCDBallotMeasureContestSourceProxy
)
from .candidatecontests import (
    OCDCandidateContestProxy,
    OCDCandidateContestPostProxy,
    OCDCandidateContestSourceProxy
)
from .candidacies import (
    OCDCandidacyProxy,
    OCDCandidacySourceProxy
)
from .elections import (
    OCDElectionProxy,
    OCDElectionIdentifierProxy,
    OCDElectionSourceProxy
)
from .parties import OCDPartyProxy
from .retentioncontests import (
    OCDRetentionContestProxy,
    OCDRetentionContestIdentifierProxy,
    OCDRetentionContestOptionProxy,
    OCDRetentionContestSourceProxy
)


__all__ = (
    "OCDBallotMeasureContestProxy",
    "OCDBallotMeasureContestIdentifierProxy",
    "OCDBallotMeasureContestOptionProxy",
    "OCDBallotMeasureContestSourceProxy",
    "OCDCandidateContestProxy",
    "OCDCandidateContestPostProxy",
    "OCDCandidateContestSourceProxy",
    "OCDCandidacyProxy",
    "OCDCandidacySourceProxy",
    "OCDElectionProxy",
    "OCDElectionIdentifierProxy",
    "OCDElectionSourceProxy",
    "OCDPartyProxy",
    "OCDRetentionContestProxy",
    "OCDRetentionContestIdentifierProxy",
    "OCDRetentionContestOptionProxy",
    "OCDRetentionContestSourceProxy"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unittests for CalAccess base model methods.
"""
from unittest import TestCase
from django.db.models.base import ModelBase
from calaccess_processed_filings.models import Form460Filing


class CalAccessBaseModelTests(TestCase):
    """
    Test for Form460Filing model.
    """
    def test_doc(self):
        """
        Confirm doc method returns a non-empty string.
        """
        doc = Form460Filing().doc()
        self.assertGreater(len(doc), 0)

    def test_field_list(self):
        """
        Confirm field_list method returns a non-empty list.
        """
        fields_list = Form460Filing().get_field_list()
        self.assertGreater(len(fields_list), 0)

    def test_db_table(self):
        """
        Confirm db_table property equals '{app_label}_{model_name}' string.
        """
        self.assertEqual(Form460Filing().db_table, 'calaccess_processed_filings_form460filing')

    def test_klass(self):
        """
        Confirm klass is an instance of ModelBase.
        """
        self.assertIsInstance(Form460Filing().klass, ModelBase)

    def test_klass_name(self):
        """
        Confirm model name equals klass_name string.
        """
        self.assertEqual('Form460Filing', Form460Filing().klass_name)

    def test_klass_group(self):
        """
        Confirm klass_group does not return None.
        """
        self.assertIsNotNone(Form460Filing().klass_name)



# Generated by Django 3.2.4 on 2021-06-11 13:22

import calaccess_processed.proxies
import calaccess_processed_elections.proxies.calaccess_scraped.base
from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('elections', '0008_auto_20181029_1527'),
        ('calaccess_scraped', '0003_auto_20210426_2334'),
        ('calaccess_raw', '0018_auto_20210426_2015'),
        ('core', '0006_merge_20200103_1432'),
    ]

    operations = [
        migrations.CreateModel(
            name='OCDBallotMeasureContestIdentifierProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.ballotmeasurecontestidentifier', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDBallotMeasureContestOptionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.ballotmeasurecontestoption', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDBallotMeasureContestProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.ballotmeasurecontest', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDBallotMeasureContestSourceProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.ballotmeasurecontestsource', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDCandidacyProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.candidacy', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDCandidacySourceProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.candidacysource', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDCandidateContestPostProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.candidatecontestpost', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDCandidateContestProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.candidatecontest', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDCandidateContestSourceProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.candidatecontestsource', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDDivisionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.division', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDElectionIdentifierProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.electionidentifier', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDElectionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.election', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDElectionSourceProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.electionsource', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDJurisdictionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.jurisdiction', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDMembershipProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.membership', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDOrganizationIdentifierProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.organizationidentifier', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDOrganizationNameProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.organizationname', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDOrganizationProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.organization', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDPartyProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.organization', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDPersonIdentifierProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.personidentifier', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDPersonNameProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.personname', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDPersonProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.person', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDPostProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('core.post', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDRetentionContestIdentifierProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.retentioncontestidentifier', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDRetentionContestOptionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.retentioncontestoption', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDRetentionContestProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.retentioncontest', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='OCDRetentionContestSourceProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('elections.retentioncontestsource', calaccess_processed.proxies.OCDProxyModelMixin),
        ),
        migrations.CreateModel(
            name='RawFilerToFilerTypeCdProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('calaccess_raw.filertofilertypecd',),
        ),
        migrations.CreateModel(
            name='ScrapedCandidateElectionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=(calaccess_processed_elections.proxies.calaccess_scraped.base.ScrapedElectionProxyMixin, 'calaccess_scraped.candidateelection'),
        ),
        migrations.CreateModel(
            name='ScrapedCandidateProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('calaccess_scraped.candidate', calaccess_processed_elections.proxies.calaccess_scraped.base.ScrapedNameMixin),
        ),
        migrations.CreateModel(
            name='ScrapedIncumbentElectionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=(calaccess_processed_elections.proxies.calaccess_scraped.base.ScrapedElectionProxyMixin, 'calaccess_scraped.incumbentelection'),
        ),
        migrations.CreateModel(
            name='ScrapedIncumbentProxy',
            fields=[
            ],
            options={
                'ordering': ['-session'],
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('calaccess_scraped.incumbent', calaccess_processed_elections.proxies.calaccess_scraped.base.ScrapedNameMixin),
        ),
        migrations.CreateModel(
            name='ScrapedPropositionElectionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=(calaccess_processed_elections.proxies.calaccess_scraped.base.ScrapedElectionProxyMixin, 'calaccess_scraped.propositionelection'),
        ),
        migrations.CreateModel(
            name='ScrapedPropositionProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('calaccess_scraped.proposition',),
            managers=[
                ('ballot_measures', django.db.models.manager.Manager()),
            ],
        ),
    ]



from csvkit import CSVKitWriter
from django.db import connection
from calaccess_campaign_browser import models
from calaccess_campaign_browser.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    help = 'Export candidates scraped from the state site'

    def handle(self, *args, **options):
        self.cursor = connection.cursor()
        sql = """
        SELECT DISTINCT
            o.name,
            o.seat,
            f.filer_id_raw,
            f.xref_filer_id,
            f.name,
            f.party
        FROM %(candidate)s as c
        INNER JOIN %(office)s as o
        ON c.office_id = o.id
        INNER JOIN %(filer)s as f
        ON c.filer_id = f.id
        """ % dict(
            candidate=models.Candidate._meta.db_table,
            office=models.Office._meta.db_table,
            filer=models.Filer._meta.db_table,
        )
        self.cursor.execute(sql)
        writer = CSVKitWriter(open("./candidates.csv", 'wb'))
        writer.writerow([
            'office_name',
            'office_seat',
            'filer_id',
            'xref_filer_id',
            'name',
            'party'
        ])
        writer.writerows(self.cursor.fetchall())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PostgreSQL database customizations.
"""
from __future__ import unicode_literals
from django.db.models import Func, IntegerField
from django.contrib.postgres.fields import JSONField
from django.contrib.postgres.operations import CreateExtension


class JSONArrayLength(Func):
    """
    Returns the length of a JSON array.
    """
    function = 'JSONB_ARRAY_LENGTH'
    output_field = IntegerField()


class JSONExtractPath(Func):
    """
    Returns JSON value pointed to by key.
    """
    template = "JSONB_EXTRACT_PATH(%(expressions)s, '%(key)s')"
    output_field = JSONField()

    def __init__(self, expression, key):
        """
        Create an instance.
        """
        super(JSONExtractPath, self).__init__(expression, key=key)


class MaxFromJSONIntegerArray(Func):
    """
    Return the maximum value of an array of integers in key.
    """
    template = "(SORT_DESC(ARRAY(SELECT JSONB_ARRAY_ELEMENTS_TEXT(%(expressions)s->'%(key)s')::int)::int[]))[1]"
    output_field = IntegerField()

    def __init__(self, expression, key):
        """
        Create an instance.
        """
        super(MaxFromJSONIntegerArray, self).__init__(expression, key=key)


class CryptoExtension(CreateExtension):
    """
    Install the pgcrypto extension to PostgresSQL.
    """
    def __init__(self):
        """
        The name of the extension to install goes here.
        """
        self.name = 'pgcrypto'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Basic configuration for the application.
"""
from __future__ import unicode_literals
import os
from django.apps import AppConfig


class CalAccessProcessedCampaignFinanceConfig(AppConfig):
    """
    Application configuration.
    """
    name = 'calaccess_processed_campaignfinance'
    verbose_name = "CAL-ACCESS processed data: Campaign finance"
    default_auto_field = 'django.db.models.AutoField'
    # Where SQL files are stored in this application
    sql_directory_path = os.path.join(os.path.dirname(__file__), 'sql')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base models for tables from the CAL-ACCESS database.
"""
from __future__ import unicode_literals

# Models
from django.db import models
from django.db.models.base import ModelBase

# Managers
from ..managers import BulkLoadSQLManager

# Text
import textwrap
from django.template.defaultfilters import capfirst


class CalAccessMetaClass(ModelBase):
    """
    A custom metaclass for our base model.

    Automatically configures Meta attributes common to all models.
    """
    def __new__(cls, name, bases, attrs):
        """
        Override the default __new__ behavior.
        """
        klass = super(CalAccessMetaClass, cls).__new__(cls, name, bases, attrs)

        # Cook up an automated verbose name for each model
        klass_str = str(klass).replace("<class 'calaccess_processed.models.", "")
        klass_group = klass_str.split(".")[0].upper()
        klass_table = capfirst(klass._meta.verbose_name_plural)
        klass_name = "{0}: {1}".format(klass_group, klass_table)

        # Insert the verbose name into each model's configuration
        klass._meta.verbose_name = klass_name
        klass._meta.verbose_name_plural = klass_name

        # Set the app_label too
        klass._meta.app_label = "calaccess_processed"

        # Finish up
        return klass


class CalAccessBaseModel(models.Model):
    """
    An abstract model with some tricks we'll reuse.
    """
    __metaclass__ = CalAccessMetaClass
    objects = BulkLoadSQLManager()

    def doc(self):
        """
        Return the model's docstring as a readable string ready to print.
        """
        if self.__doc__.startswith(self.klass_name):
            return ''
        return textwrap.dedent(self.__doc__).strip()

    @property
    def db_table(self):
        """
        Return the model's database table name as a string.
        """
        return self._meta.db_table

    @property
    def klass(self):
        """
        Return the model class itself.
        """
        return self.__class__

    @property
    def klass_name(self):
        """
        Return the name of the model class.
        """
        return self.__class__.__name__

    @property
    def klass_group(self):
        """
        Return the name of the model's group, as determined by its submodule.
        """
        return str(self.__class__).split(".")[2]

    def get_field_list(self):
        """
        Return all the fields on the model as a list.
        """
        return self._meta.fields

    class Meta:
        """
        Meta model options.
        """
        abstract = True
        app_label = 'calaccess_processed'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Filing-related models, managers and mixins.
"""
import requests
from time import sleep
from calaccess_processed.models.base import CalAccessBaseModel
from calaccess_processed_filings.managers import FilingsManager


class FilingBaseModel(CalAccessBaseModel):
    """
    Base model for all the filings models.
    """
    objects = FilingsManager()

    class Meta:
        """
        Meta model options.
        """
        abstract = True
        app_label = 'calaccess_processed_filings'

    @property
    def pdf_url(self):
        """
        Returns the url for pdf of the most recent version of the CAL-ACCESS filing.
        """
        # If it's a Filing, it will have an amendment count. If it's a Version it will have an amend_id.
        try:
            amendid = getattr(self, 'amendment_count')
        except AttributeError:
            amendid = getattr(self, 'amend_id')
        return 'http://cal-access.sos.ca.gov/PDFGen/pdfgen.prg?filingid={}&amendid={}'.format(self.filing_id, amendid)

    @property
    def has_pdf(self):
        """
        Make a HEAD request filing's pdf, return True if response status code is 200.

        Pause for half a second before making the request.
        """
        sleep(0.5)
        r = requests.head(self.pdf_url)
        return r.status_code == 200



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for OCD Filing related models..
"""
from __future__ import unicode_literals
from opencivicdata.campaign_finance.models import (
    Committee,
    CommitteeType,
    CommitteeIdentifier,
    CommitteeName,
    CommitteeSource,
)
from calaccess_processed.proxies import OCDProxyModelMixin
from calaccess_processed_campaignfinance.managers import (
    CampaignFinanceManager,
    OCDCommitteeManager,
    OCDCommitteeIdentifierManager,
    OCDCommitteeNameManager,
    OCDCommitteeTypeManager
)

# Logging
import logging
logger = logging.getLogger(__name__)


class OCDCommitteeProxy(Committee, OCDProxyModelMixin):
    """
    Proxy of the OCD Committee model.
    """
    objects = OCDCommitteeManager()

    @property
    def calaccess_filer_id(self):
        """
        Returns the committee's CAL-ACCESS filer id.
        """
        return self.identifiers.get(scheme="calaccess_filer_id")

    @property
    def calaccess_filer_url(self):
        """
        Returns the URL of the committee's detail page on the CAL-ACCESS website.
        """
        url_template = "http://cal-access.sos.ca.gov/Campaign/Committees/Detail.aspx?id={}"
        return url_template.format(self.calaccess_filer_id.identifier)

    @property
    def filing_proxies(self):
        """
        A QuerySet of OCDCandidateContestProxy for the election.
        """
        from .filings import OCDFilingProxy
        return OCDFilingProxy.objects.filter(filer=self)

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDCommitteeIdentifierProxy(CommitteeIdentifier, OCDProxyModelMixin):
    """
    Proxy of the OCD CommitteeIdentifier model.
    """
    objects = OCDCommitteeIdentifierManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDCommitteeNameProxy(CommitteeName, OCDProxyModelMixin):
    """
    Proxy of the OCD CommitteeName model.
    """
    objects = OCDCommitteeNameManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDCommitteeSourceProxy(CommitteeSource, OCDProxyModelMixin):
    """
    Proxy of the OCD CommitteeSource model.
    """
    objects = CampaignFinanceManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDCommitteeTypeProxy(CommitteeType, OCDProxyModelMixin):
    """
    Proxy of the OCD CommitteeSource model.
    """
    objects = OCDCommitteeTypeManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom manager for the OCD Person model.
"""
from __future__ import unicode_literals
from calaccess_processed.managers import BulkLoadSQLManager


class OCDPersonManager(BulkLoadSQLManager):
    """
    A custom manager for working with the OCD Person model.
    """
    def get_by_filer_id(self, filer_id):
        """
        Returns a Person object linked to a CAL-ACCESS filer_id, if it exists.
        """
        return self.get_queryset().get(
            identifiers__scheme='calaccess_filer_id',
            identifiers__identifier=filer_id,
        )

    def get_or_create_from_calaccess(self, candidate_name_dict, candidate_filer_id=None):
        """
        Create a Person object using data from the CAL-ACCESS database and scrape.

        If a filer_id is provided, first attempt to lookup the Person by filer_id.
        If matched, and the provided name doesn't match the current name of the Person
        and isn't included in the other names of the Person, add it as an other_name.

        If the person doesn't exist (or the filer_id is not provided), create a
        new Person.

        Returns a tuple (Person object, created), where created is a boolean
        specifying whether a Person was created.
        """
        # If there is a filer_id, try to go that way
        if candidate_filer_id:
            try:
                person = self.get_by_filer_id(candidate_filer_id)
            except self.model.DoesNotExist:
                pass
            else:
                # If we find a match, make sure it has this name variation logged
                person.add_other_name(candidate_name_dict['name'], 'Matched on calaccess_filer_id')
                # Then pass it out.
                return person, False

        # Otherwise create a new one
        person, person_created = self.get_or_create(**candidate_name_dict)

        # If there's a filer_id, add it on
        if candidate_filer_id:
            person.add_filer_id(candidate_filer_id)

        # Pass it back
        return person, person_created



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load the OCD Membership model with data from the scraped Incumbent model.
"""
from django.db.models import IntegerField
from django.db.models.functions import Cast
from opencivicdata.core.models import Membership
from calaccess_processed.management.commands import CalAccessCommand
from opencivicdata.elections.models import Candidacy, CandidateContest
from calaccess_processed_elections.proxies import OCDMembershipProxy, ScrapedIncumbentProxy


class Command(CalAccessCommand):
    """
    Load the OCD Membership model with data from the scraped Incumbent model.
    """
    help = 'Load the OCD Membership model with data from the scraped Incumbent model'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)
        self.header("Loading Incumbent Office Holders")
        self.load()
        self.set_end_dates()
        if Candidacy.objects.exists():
            self.set_incumbent_candidacies()
        self.success("Done!")

    def load(self):
        """
        Load OCD Election, Membership and related models with data scraped from CAL-ACCESS website.
        """
        for incumbent in ScrapedIncumbentProxy.objects.all():
            membership, created = OCDMembershipProxy.objects.get_or_create_from_calaccess(
                incumbent
            )
            if created and self.verbosity > 2:
                self.log(' Created new Membership: %s' % membership)
            # Handle start_date on membershipbership
            if created or membership.start_date == '':
                membership.start_date = incumbent.session
                membership.save()
            else:
                # increment start year down
                start = int(membership.start_date)
                if start > incumbent.session:
                    membership.start_date = incumbent.session
                    membership.save()

    def set_end_dates(self):
        """
        Set the end_date for each Membership based on the start_date of each successor.
        """
        for member in Membership.objects.all():
            # Each member's end year should be the start year of their successor.
            # Successor is the member in the same post with the earliest
            # start year greater than the incumbent's start year
            if member.start_date == '':
                start_year = 0
            else:
                start_year = int(member.start_date)

            successor_q = Membership.objects.exclude(
                start_date='',
            ).annotate(
                start_year=Cast('start_date', IntegerField()),
            ).filter(
                start_year__gt=start_year,
                post=member.post,
            ).order_by('start_year')

            if successor_q.exists():
                member.end_date = int(successor_q[0].start_date)
                member.save()

    def set_incumbent_candidacies(self):
        """
        Set is_incumbent for candidacies within each member's start/end years.
        """
        # For every one of the member's candidacies for the office where
        # the election of the contest happens after the start year
        # but before the end year, if it exists, mark as incumbent
        for member in Membership.objects.all():
            candidacies_q = member.person.candidacies.filter(
                contest__election__date__year__gt=int(member.start_date),
                post=member.post,
            ).exclude(is_incumbent=True)
            # Handle blank member end_date values
            if member.end_date != '':
                member_end_year = int(member.end_date)
                candidacies_q = candidacies_q.filter(
                    contest__election__date__year__lte=member_end_year
                )

            if candidacies_q.exists():
                rows = candidacies_q.update(is_incumbent=True)
                if self.verbosity > 2:
                    self.log(
                        ' {0} identified as incumbent in {1} contests'.format(
                            member.person.name,
                            rows,
                        )
                    )
        # loop over all contest with an incumbent candidate
        contests_q = CandidateContest.objects.filter(
            candidacies__is_incumbent=True
        )
        for contest in contests_q.all():
            # set is_incumbent False for all other candidacies in contest
            contest.candidacies.exclude(
                is_incumbent=True
            ).update(is_incumbent=False)

        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Abstract base models for campaign finance-related filings and transactions.
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel


class CampaignContributionBase(FilingBaseModel):
    """
    Abstract base model for contributions received or made by campaign filers.

    These transactions are itemized on Schedules A, C and I of Form 460
    filings and stored in the RCPT_CD table.
    """
    line_item = models.IntegerField(
        verbose_name='line item',
        null=False,
        help_text='Line number of the filing form where the contribution is '
                  'itemized (from RCPT_CD.LINE_ITEM)',
    )
    date_received = models.DateField(
        verbose_name='date received',
        null=True,
        help_text='Date the contribution was received (from RCPT_CD.'
                  'RCPT_DATE)'
    )
    date_received_thru = models.DateField(
        verbose_name='date received thru',
        null=True,
        help_text='End date for late contributions received over a range of '
                  'days (from RCPT_CD.DATE_THRU)',
    )
    TRANSACTION_TYPE_CHOICES = (
        ('F', 'Forgiven Loan'),
        ('I', 'Intermediary'),
        ('R', 'Returned (Negative Amount?)'),
        ('T', 'Third Party Repayment'),
        ('X', 'Transfer'),
        ('', 'Unknown'),
        ('INC16168', 'INC16168'),
    )
    transaction_type = models.CharField(
        verbose_name='transaction type',
        max_length=255,
        choices=TRANSACTION_TYPE_CHOICES,
        help_text='Type of transaction (from RCPT_CD.TRAN_TYPE)',
    )
    transaction_id = models.CharField(
        verbose_name='transaction id',
        max_length=20,
        help_text='Identifies a unique transaction across versions of the a '
                  'given Form 460 filing (from RCPT_CD.TRAN_ID)'
    )
    memo_reference_number = models.CharField(
        verbose_name='memo reference number',
        max_length=20,
        blank=True,
        help_text='Reference number for the memo attached to the contribution '
                  '(from RCPT_CD.MEMO_REFNO)',
    )
    CONTRIBUTOR_CODE_CHOICES = (
        ('COM', 'Committee'),
        ('IND', 'Individual'),
        ('OFF', 'Officer'),
        ('OTH', 'Other'),
        ('PTY', 'Political Party'),
        ('RCP', 'Recipient committee'),
        ('SCC', 'Small Contributor Committee'),
    )
    contributor_code = models.CharField(
        verbose_name='contributor code',
        max_length=3,
        blank=True,
        choices=CONTRIBUTOR_CODE_CHOICES,
        help_text='Code describing the contributor (from RCPT_CD.ENTITY_CD)',
    )
    contributor_committee_id = models.CharField(
        verbose_name='contributor committee id',
        max_length=9,
        blank=True,
        help_text="Contributor's filer identification number, if it is a "
                  "committee (from RCPT_CD.CMTE_ID)",
    )
    contributor_title = models.CharField(
        verbose_name='contributor title',
        max_length=10,
        blank=True,
        help_text='Name title of the contributor (from RCPT_CD.CTRIB_NAMT)',
    )
    contributor_lastname = models.CharField(
        verbose_name='contributor lastname',
        max_length=200,
        blank=True,
        help_text='Last name of the contributor or business name (from '
                  'RCPT_CD.CTRIB_NAML)',
    )
    contributor_firstname = models.CharField(
        verbose_name='contributor firstname',
        max_length=45,
        help_text='First name of the contributor (from RCPT_CD.CTRIB_NAMF)',
    )
    contributor_name_suffix = models.CharField(
        verbose_name='contributor name suffix',
        max_length=10,
        blank=True,
        help_text='Name suffix of the contributor (from RCPT_CD.CTRIB_NAMS)',
    )
    contributor_city = models.CharField(
        verbose_name='contributor city',
        max_length=30,
        blank=True,
        help_text='City of the contributor (from RCPT_CD.CTRIB_CITY)',
    )
    contributor_state = models.CharField(
        verbose_name='contributor state',
        max_length=2,
        blank=True,
        help_text='State of the contributor (from RCPT_CD.CTRIB_ST)',
    )
    contributor_zip = models.CharField(
        verbose_name='contributor zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'contributor (from RCPT_CD.CTRIB_ZIP4)',
    )
    contributor_employer = models.CharField(
        verbose_name='contributor employer',
        max_length=200,
        blank=True,
        help_text='Employer of the contributor (from RCPT_CD.CTRIB_EMP)',
    )
    contributor_occupation = models.CharField(
        verbose_name='contributor occupation',
        max_length=60,
        blank=True,
        help_text='Occupation of the contributor (from RCPT_CD.CTRIB_OCC)',
    )
    contributor_is_self_employed = models.BooleanField(
        verbose_name='contributor is self employed',
        default=False,
        help_text='Indicates whether or not the contributor is self-employed'
                  '(from RCPT_CD.CTRIB_SELF)',
    )
    intermediary_committee_id = models.CharField(
        verbose_name='intermediary committee id',
        blank=True,
        max_length=9,
        help_text="Intermediary's filer identification number, if it is a "
                  "committee (from RCPT_CD.INTR_CMTEID)",
    )
    intermediary_title = models.CharField(
        verbose_name='intermediary title',
        max_length=10,
        blank=True,
        help_text='Name title of the intermediary (from RCPT_CD.INTR_NAMT)',
    )
    intermediary_lastname = models.CharField(
        verbose_name='intermediary lastname',
        max_length=200,
        blank=True,
        help_text='Last name of the intermediary or business name (from '
                  'RCPT_CD.INTR_NAML)',
    )
    intermediary_firstname = models.CharField(
        verbose_name='intermediary firstname',
        max_length=45,
        help_text='First name of the intermediary (from RCPT_CD.INTR_NAMF)',
    )
    intermediary_name_suffix = models.CharField(
        verbose_name='intermediary name suffix',
        max_length=10,
        blank=True,
        help_text='Name suffix of the intermediary (from RCPT_CD.INTR_NAMS)',
    )
    intermediary_city = models.CharField(
        verbose_name='intermediary city',
        max_length=30,
        blank=True,
        help_text='City of the intermediary (from RCPT_CD.INTR_CITY)',
    )
    intermediary_state = models.CharField(
        verbose_name='intermediary state',
        max_length=2,
        blank=True,
        help_text='State of the intermediary (from RCPT_CD.INTR_ST)',
    )
    intermediary_zip = models.CharField(
        verbose_name='intermediary zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'intermediary (from RCPT_CD.INTR_ZIP4)',
    )
    intermediary_employer = models.CharField(
        verbose_name='intermediary employer',
        max_length=200,
        blank=True,
        help_text='Employer of the intermediary (from RCPT_CD.INTR_EMP)',
    )
    intermediary_occupation = models.CharField(
        verbose_name='intermediary occupation',
        max_length=60,
        blank=True,
        help_text='Occupation of the intermediary (from RCPT_CD.INTR_OCC)',
    )
    intermediary_is_self_employed = models.BooleanField(
        verbose_name='intermediary is self employed',
        default=False,
        help_text='(from S497_CD.INTR_SELF)',
    )
    cumulative_ytd_amount = models.DecimalField(
        decimal_places=2,
        max_digits=14,
        null=True,
        help_text="Cumulative year-to-date amount given by the contributor "
                  "as of the given Form 460 filing (from RCPT_CD.CUM_YTD)",
    )
    cumulative_election_amount = models.DecimalField(
        decimal_places=2,
        max_digits=14,
        null=True,
        help_text="For filers subject to contribution limits, cumulative "
                  "amount given by the contributor during the election "
                  "cycle as of the given Form 460 filing (from RCPT_CD."
                  "CUM_OTH)"
    )

    class Meta:
        """
        Model options.
        """
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Find and merge OCD Person records that share a name and CandidateContest.
"""
from django.db.models import Count
from calaccess_processed_elections.merge import merge_persons
from calaccess_processed.management.commands import CalAccessCommand
from calaccess_processed_elections.proxies import OCDCandidateContestProxy


class Command(CalAccessCommand):
    """
    Find and merge OCD Person records that share a name and CandidateContest.
    """
    help = 'Find and merge OCD Person records that share a name and CandidateContest'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        self.header("Merging Persons in same Contest with shared name")

        # Loop over all CandidateContests
        for contest in OCDCandidateContestProxy.objects.all():
            # handle groups by candidate_name
            for group_q in self.group_by_candidate_name(contest.candidacies):
                self.handle_group(group_q, field_name='candidate_name')
            # handle groups by person_name
            for group_q in self.group_by_person_name(contest.candidacies):
                self.handle_group(group_q, field_name='person_name')
            # handle groups by person_other_name
            for group_q in self.group_by_other_name(contest.candidacies):
                self.handle_group(group_q, field_name='other_name')

        self.success("Done!")

    def group_by_candidate_name(self, candidacies_q):
        """
        Return a list of QuerySets for Candidacies with the same candidate_name.
        """
        q = candidacies_q.values('candidate_name').annotate(
            row_count=Count('id'),
        ).order_by().filter(row_count__gt=1)

        results = []

        for i in q.all():
            results.append(
                candidacies_q.filter(
                    candidate_name=i['candidate_name']
                ).order_by()
            )

        return results

    def group_by_person_name(self, candidacies_q):
        """
        Return a list of QuerySets for Candidacies with the same Person name.
        """
        q = candidacies_q.values('person__name').annotate(
            row_count=Count('id'),
        ).order_by().filter(row_count__gt=1)

        results = []

        for i in q.all():
            results.append(
                candidacies_q.filter(
                    person__name=i['person__name']
                ).order_by()
            )

        return results

    def group_by_other_name(self, candidacies_q):
        """
        Return a list of QuerySets for Candidacies with shared other_name.
        """
        # get the distinct count of person_ids for each other_name
        # linked to a person who's linked to one of the contest's candidacies
        q = candidacies_q.values('person__other_names__name').annotate(
            row_count=Count('person_id', distinct=True),
        ).order_by().filter(row_count__gt=1)

        results = []

        for i in q.all():
            if i['person__other_names__name']:
                results.append(
                    candidacies_q.filter(
                        person__other_names__name=i['person__other_names__name']
                    ).order_by()
                )

        return results

    def group_by_party(self, candidacies_q):
        """
        Return a list of QuerySets for Candidacies with the same party.
        """
        q = candidacies_q.values('party').annotate(
            row_count=Count('id'),
        ).order_by().filter(row_count__gt=1)

        results = []

        for i in q.all():
            results.append(
                candidacies_q.filter(
                    party=i['party']
                ).order_by()
            )

        return results

    def get_group_filer_id_count(self, candidacies_q):
        """
        Return the count of distinct filer_ids in the Candidacy QuerySet.
        """
        return candidacies_q.filter(
            person__identifiers__scheme='calaccess_filer_id'
        ).distinct('person__identifiers__identifier').count()

    def log_merged_persons(self, group_q, field_name):
        """
        Log the persons in group_q who will be merged.
        """
        self.log(
            'Merging {0} persons grouped by {1} in {2}'.format(
                group_q.count(),
                field_name,
                group_q.all()[0].contest,
            )
        )
        for c in group_q.all():
            self.log(' - {} ({})'.format(c.person, c.person_id))

    def handle_group(self, group_q, field_name=None):
        """
        Handle merging of candidates in group_q.

        Optional field_name is a string naming the field used to form the group.
        """
        # if there isn't more than one party and more than one filer_id
        if (
            group_q.distinct('party').exclude(party__isnull=True).count() <= 1
            and self.get_group_filer_id_count(group_q) <= 1
        ):
            if self.verbosity > 2:
                self.log_merged_persons(group_q, field_name)
            # merge
            merge_persons([c.person for c in group_q.all()])
        # handle multiple parties in the group
        elif group_q.distinct('party').count() > 1:
            # for each group with the same party in the group
            for party_group_q in self.group_by_party(group_q):
                # if there's only one filer_id
                if self.get_group_filer_id_count(party_group_q) <= 1:
                    if self.verbosity > 2:
                        self.log_merged_persons(party_group_q, field_name)
                    # merge the group
                    merge_persons(
                        [c.person for c in party_group_q.all()]
                    )
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Basic configuration for the application.
"""
from __future__ import unicode_literals
import os
import collections
from django.apps import AppConfig


class CalAccessProcessedFlatfilesConfig(AppConfig):
    """
    Application configuration.
    """
    name = 'calaccess_processed_flatfiles'
    verbose_name = "CAL-ACCESS processed data: Flat files"
    default_auto_field = 'django.db.models.AutoField'

    # Where SQL files are stored in this application
    sql_directory_path = os.path.join(os.path.dirname(__file__), 'sql')

    def get_flat_names_list(self):
        """
        Returns a list of all of the flatfile names.
        """
        return list(self.get_flat_proxy_lookup().keys())

    def get_flat_proxy_list(self):
        """
        Returns a list of all the flatfile proxies.
        """
        return list(self.get_flat_proxy_lookup().values())

    def get_flat_proxy_lookup(self):
        """
        Returns a dictionary crosswalk between flatfile names and proxies.
        """
        from . import proxies
        return collections.OrderedDict({
            'Candidates': proxies.OCDFlatCandidacyProxy,
            'BallotMeasures': proxies.OCDFlatBallotMeasureContestProxy,
            'RecallMeasures': proxies.OCDFlatRetentionContestProxy
        })



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utilities for more quickly loading bulk data.
"""
from __future__ import unicode_literals
import os
import logging
from django.apps import apps
from postgres_copy.managers import CopyManager
logger = logging.getLogger(__name__)


class BulkLoadSQLManager(CopyManager):
    """
    Utilities for more quickly loading bulk data into a model with custom SQL.
    """
    app_name = "calaccess_processed"

    def get_sql(self):
        """
        Return string of raw sql for loading the model.
        """
        raise NotImplementedError

    def get_sql_path(self, file_name):
        """
        Return the full path with extenstion to file_name.
        """
        sql_path = apps.get_app_config(self.app_name).sql_directory_path
        return os.path.join(sql_path, '%s.sql' % file_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unittests for management commands.
"""
from django.test import TestCase
from calaccess_processed.management.commands import CalAccessCommand


class BaseCommandsTest(TestCase):
    """
    Tests to run with no data loaded.
    """
    def test_base_command(self):
        """
        Test options on base commands.
        """
        c = CalAccessCommand()
        c.handle()
        c.header("")
        c.log("")
        c.success("")
        c.warn("")
        c.failure("")
        c.duration()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Abstract base models for campaign finance-related filings and transactions.
"""
from .contribution import CampaignContributionBase
from .expenditure import (
    CampaignExpenditureItemBase,
    CampaignExpenditureSubItemBase
)
from .filing import CampaignFinanceFilingBase
from .loan import (
    CampaignLoanItemBase,
    CampaignLoanReceivedItemBase,
    CampaignLoanMadeItemBase,
)


__all__ = (
    "CampaignContributionBase",
    "CampaignExpenditureItemBase",
    "CampaignExpenditureSubItemBase",
    "CampaignFinanceFilingBase",
    "CampaignLoanItemBase",
    "CampaignLoanReceivedItemBase",
    "CampaignLoanMadeItemBase",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the models from submodules and thread them together.
"""
from .base import CalAccessMetaClass, CalAccessBaseModel
from .tracking import (
    ProcessedDataVersion,
    ProcessedDataFile,
    ProcessedDataZip
)

__all__ = (
    'CalAccessMetaClass',
    'CalAccessBaseModel',
    'ProcessedDataVersion',
    'ProcessedDataFile',
    'ProcessedDataZip'
)



# Generated by Django 2.0.4 on 2018-04-27 00:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0008_auto_20180426_2358'),
    ]

    operations = [
        migrations.CreateModel(
            name='Form461Part5Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('line_item', models.IntegerField(help_text='Line number of the filing form where the payment is itemized (from EXPN_CD.LINE_ITEM)', verbose_name='line item')),
                ('payee_code', models.CharField(blank=True, choices=[('BNM', "Ballot measure's name/title"), ('CAO', 'Candidate/officeholder'), ('COM', 'Committee'), ('IND', 'Individual'), ('MBR', 'Member of Associaton'), ('OFF', 'Officer'), ('OTH', 'Other'), ('PTY', 'Political Party'), ('RCP', 'Recipient committee'), ('SCC', 'Small Contributor Committee'), ('???', 'Unknown value')], help_text='Code describing the payee (from EXPN_CD.ENTITY_CD)', max_length=3, verbose_name='payee code')),
                ('payee_committee_id', models.CharField(blank=True, help_text="Payee's filer identification number, if it is a committee (from EXPN_CD.CMTE_ID)", max_length=9, verbose_name='payee committee id')),
                ('payee_title', models.CharField(blank=True, help_text='Name title of the payee (from EXPN_CD.PAYEE_NAMT)', max_length=10, verbose_name='payee title')),
                ('payee_lastname', models.CharField(blank=True, help_text='Last name of the payee or business name (from EXPN_CD.PAYEE_NAML)', max_length=200, verbose_name='payee lastname')),
                ('payee_firstname', models.CharField(help_text='First name of the payee (from EXPN_CD.PAYEE_NAMF)', max_length=45, verbose_name='payee firstname')),
                ('payee_name_suffix', models.CharField(blank=True, help_text='Name suffix of the payee (from EXPN_CD.PAYEE_NAMS)', max_length=10, verbose_name='payee name suffix')),
                ('payee_city', models.CharField(blank=True, help_text='City of the payee (from EXPN_CD.PAYEE_CITY)', max_length=30, verbose_name='payee city')),
                ('payee_state', models.CharField(blank=True, help_text='State of the payee (from EXPN_CD.PAYEE_ST)', max_length=2, verbose_name='payee state')),
                ('payee_zip', models.CharField(blank=True, help_text='Zip code (usually zip5, sometimes zip9) of the payee (from EXPN_CD.PAYEE_ZIP4)', max_length=10, verbose_name='payee zip')),
                ('treasurer_title', models.CharField(blank=True, help_text="Name title of the payee committee's treasurer (from EXPN_CD.TRES_NAMT)", max_length=10, verbose_name='treasurer title')),
                ('treasurer_lastname', models.CharField(blank=True, help_text="Last name of the payee committee's treasurer (from EXPN_CD.TRES_NAML)", max_length=200, verbose_name='treasurer lastname')),
                ('treasurer_firstname', models.CharField(help_text="First name of the payee committee's treasurer (from EXPN_CD.TRES_NAMF)", max_length=45, verbose_name='treasurer firstname')),
                ('treasurer_name_suffix', models.CharField(blank=True, help_text="Name suffix of the payee committee's treasurer (from EXPN_CD.TRES_NAMS)", max_length=10, verbose_name='treasurer name suffix')),
                ('treasurer_city', models.CharField(blank=True, help_text="City of the payee committee's treasurer (from EXPN_CD.TRES_CITY)", max_length=30, verbose_name='treasurer city')),
                ('treasurer_state', models.CharField(blank=True, help_text="State of the payee committee's treasurer (from EXPN_CD.TRES_ST)", max_length=2, verbose_name='treasurer state')),
                ('treasurer_zip', models.CharField(blank=True, help_text="Zip code (usually zip5, sometimes zip9) of the payee committee's treasurer (from EXPN_CD.TRES_ZIP4)", max_length=10, verbose_name='treasurer zip')),
                ('payment_code', models.CharField(blank=True, choices=[('CMP', 'Campaign paraphernalia/miscellaneous'), ('CNS', 'Campaign consultants'), ('CTB', 'Contribution (if nonmonetary, explain)*'), ('CVC', 'Civic donations'), ('FIL', 'Candidate filing/ballot feeds'), ('FND', 'Fundraising events'), ('IKD', 'In-kind contribution (nonmonetary)'), ('IND', 'Independent expenditure supporting/opposing others (explain)*'), ('LEG', 'Legal defense'), ('LIT', 'Campaign literature and mailings'), ('LON', 'Loan'), ('MBR', 'Member communications'), ('MON', 'Monetary contribution'), ('MTG', 'Meetings and appearances'), ('OFC', 'Office expenses'), ('PET', 'Petition circulating'), ('PHO', 'Phone banks'), ('POL', 'Polling and survey research'), ('POS', 'Postage, delivery and messenger services'), ('PRO', 'Professional services (legal, accounting)'), ('PRT', 'Print ads'), ('RAD', 'Radio airtime and production costs'), ('RFD', 'Returned contributions'), ('SAL', 'Campaign workers salaries'), ('TEL', 'T.V. or cable airtime and production costs'), ('TRC', 'Candidate travel, lodging and meals (explain)'), ('TRS', 'Staff/spouse travel, lodging and meals (explain)'), ('TSF', 'Transfer between committees of the same candidate/sponsor'), ('VOT', 'Voter registration'), ('WEB', 'Information technology costs (internet, e-mail)'), ('???', 'Unknown value')], help_text='Code describing the payment (from EXPN_CD.EXPN_CODE)', max_length=3, verbose_name='payment code')),
                ('payment_description', models.CharField(blank=True, help_text='Purpose of payment and/or description/explanation (from EXPN_CD.EXPN_DSCR)', max_length=400, verbose_name='payment description')),
                ('amount', models.DecimalField(decimal_places=2, help_text='Amount paid to the payee in the period covered by the filing (from EXPN_CD.AMOUNT)', max_digits=14, verbose_name='amount')),
                ('cumulative_ytd_amount', models.DecimalField(decimal_places=2, help_text="Cumulative year-to-date amount given or spent by the filer in support or opposition of the candidate or ballot measure as of the Form 460's filing date (from EXPN_CD.CUM_YTD)", max_digits=14, null=True)),
                ('expense_date', models.DateField(help_text='Date or expense (from EXPN_CD.EXPN_DATE)', null=True, verbose_name='expense date')),
                ('check_number', models.CharField(blank=True, help_text='Optional check number for the payment made by the campaign filer (from EXPN_CD.EXPN_CHKNO)', max_length=20, verbose_name='expense check number')),
                ('support_oppose_code', models.CharField(blank=True, choices=[('S', 'Support'), ('O', 'Oppose'), ('?', 'Unknown value')], help_text='If applicable, code indicating whether the payment went toward supporting or opposing a candidate/ballot measure (from EXPN_CD.SUP_OPP_CD)', max_length=1, verbose_name='support oppose code')),
                ('ballot_measure_jurisdiction', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a ballot measure, the jurisdiction subject to the ballot measure (from EXPN_CD.BAL_JURIS)', max_length=40, verbose_name='ballot measure jurisdiction')),
                ('ballot_measure_name', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a ballot measure, name of the ballot measure (from EXPN_CD.BAL_NAME  or EXPN_CD.CAND_NAML)', max_length=200, verbose_name='ballot measure name')),
                ('ballot_measure_num', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a ballot measure, ballot number or letter (from EXPN_CD.BAL_NUM)', max_length=7, verbose_name='ballot measure number')),
                ('candidate_title', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, name title of the candidate (from EXPN_CD.CAND_NAMT)', max_length=10, verbose_name='candidate title')),
                ('candidate_lastname', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, last name of the candidate or business name (from EXPN_CD.CAND_NAML)', max_length=200, verbose_name='candidate lastname')),
                ('candidate_firstname', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, first name of the candidate (from EXPN_CD.CAND_NAMF)', max_length=45, verbose_name='candidate firstname')),
                ('candidate_name_suffix', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, name suffix of the candidate (from EXPN_CD.CAND_NAMS)', max_length=10, verbose_name='candidate name suffix')),
                ('candidate_jurisdiction_code', models.CharField(blank=True, choices=[('ASM', 'Assembly District'), ('BOE', 'Board of Equalization District'), ('CIT', 'City'), ('CTY', 'County'), ('LOC', 'Local'), ('OTH', 'Other'), ('SEN', 'Senate District'), ('STW', 'Statewide'), ('???', 'Statewide')], help_text='If the payment went toward supporting/opposing a candidate,code indicating the jurisdiction of the office (from EXPN_CD.JURIS_CD)', max_length=3, verbose_name='candidate jurisdiction')),
                ('candidate_jurisdiction_description', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a county, city or local candidate, full description of the office (from EXPN_CD.JURIS_DSCR)', max_length=40, verbose_name='candidate jurisdiciton description')),
                ('candidate_district', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate,for state senate, assembly or local board of education, thedistrict of the office (from EXPN_CD.DIST_NO)', max_length=3, verbose_name='candidate district')),
                ('office_sought_held', models.CharField(blank=True, choices=[('S', 'SOUGHT'), ('H', 'HELD')], help_text='If the payment went toward supporting/opposing a candidate,code indicating if the candidate is seeking or currently holds the office (from EXPN_CD.OFF_S_H_CD)', max_length=1, verbose_name='office sought or held')),
                ('office_code', models.CharField(blank=True, choices=[('APP', 'State Appellate Court Justice'), ('ASM', 'State Assembly Person'), ('ASR', 'Assessor'), ('ATT', 'Attorney General'), ('BED', 'Board of Education'), ('BOE', 'Board of Equalization Member'), ('BSU', 'Board of Supervisors'), ('CAT', 'City Attorney'), ('CCB', 'Community College Board'), ('CCM', 'City Council Member'), ('CON', 'State Controller'), ('COU', 'County Counsel'), ('CSU', 'County Supervisor'), ('CTR', 'Local Controller'), ('DAT', 'District Attorney'), ('GOV', 'Governor'), ('INS', 'Insurance Commissioner'), ('LTG', 'Lieutenant Governor'), ('MAY', 'Mayor'), ('OTH', 'Other'), ('PDR', 'Public Defender'), ('PER', 'Public Employees Retirement System'), ('PLN', 'Planning Commissioner'), ('SCJ', 'Superior Court Judge'), ('SEN', 'State Senator'), ('SHC', 'Sheriff-Coroner'), ('SOS', 'Secretary of State'), ('SPM', 'Supreme Court Justice'), ('SUP', 'Superintendent of Public Instruction'), ('TRE', 'State Treasurer'), ('TRS', 'Local Treasurer'), ('???', 'Unknown value')], help_text='If the payment went toward supporting/opposing a candidate,code describing the office (from EXPN_CD.OFFICE_CD)', max_length=3, verbose_name='office code')),
                ('office_description', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate,description of the office (from EXPN_CD.OFFIC_DSCR)', max_length=40, verbose_name='office description')),
                ('transaction_id', models.CharField(help_text='Identifies a unique transaction across versions of the a given Form 460 filing (from EXPN_CD.TRAN_ID)', max_length=20, verbose_name='transaction id')),
                ('memo_reference_number', models.CharField(blank=True, help_text="A value assigned by the filer which refers to the item'sfootnote in the TEXT_MEMO_CD table (from EXPN_CD.MEMO_REFNO)", max_length=20, verbose_name='memo reference number')),
                ('filing', models.ForeignKey(help_text='Foreign key referring to the Form 461 on which the payment was reported (from EXPN_CD.FILING_ID)', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='part_5_items', to='calaccess_processed_filings.Form461Filing')),
            ],
            options={
                'verbose_name': 'Form 461 (Campaign Disclosure) Part 5 item',
            },
        ),
        migrations.CreateModel(
            name='Form461Part5ItemVersion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('line_item', models.IntegerField(help_text='Line number of the filing form where the payment is itemized (from EXPN_CD.LINE_ITEM)', verbose_name='line item')),
                ('payee_code', models.CharField(blank=True, choices=[('BNM', "Ballot measure's name/title"), ('CAO', 'Candidate/officeholder'), ('COM', 'Committee'), ('IND', 'Individual'), ('MBR', 'Member of Associaton'), ('OFF', 'Officer'), ('OTH', 'Other'), ('PTY', 'Political Party'), ('RCP', 'Recipient committee'), ('SCC', 'Small Contributor Committee'), ('???', 'Unknown value')], help_text='Code describing the payee (from EXPN_CD.ENTITY_CD)', max_length=3, verbose_name='payee code')),
                ('payee_committee_id', models.CharField(blank=True, help_text="Payee's filer identification number, if it is a committee (from EXPN_CD.CMTE_ID)", max_length=9, verbose_name='payee committee id')),
                ('payee_title', models.CharField(blank=True, help_text='Name title of the payee (from EXPN_CD.PAYEE_NAMT)', max_length=10, verbose_name='payee title')),
                ('payee_lastname', models.CharField(blank=True, help_text='Last name of the payee or business name (from EXPN_CD.PAYEE_NAML)', max_length=200, verbose_name='payee lastname')),
                ('payee_firstname', models.CharField(help_text='First name of the payee (from EXPN_CD.PAYEE_NAMF)', max_length=45, verbose_name='payee firstname')),
                ('payee_name_suffix', models.CharField(blank=True, help_text='Name suffix of the payee (from EXPN_CD.PAYEE_NAMS)', max_length=10, verbose_name='payee name suffix')),
                ('payee_city', models.CharField(blank=True, help_text='City of the payee (from EXPN_CD.PAYEE_CITY)', max_length=30, verbose_name='payee city')),
                ('payee_state', models.CharField(blank=True, help_text='State of the payee (from EXPN_CD.PAYEE_ST)', max_length=2, verbose_name='payee state')),
                ('payee_zip', models.CharField(blank=True, help_text='Zip code (usually zip5, sometimes zip9) of the payee (from EXPN_CD.PAYEE_ZIP4)', max_length=10, verbose_name='payee zip')),
                ('treasurer_title', models.CharField(blank=True, help_text="Name title of the payee committee's treasurer (from EXPN_CD.TRES_NAMT)", max_length=10, verbose_name='treasurer title')),
                ('treasurer_lastname', models.CharField(blank=True, help_text="Last name of the payee committee's treasurer (from EXPN_CD.TRES_NAML)", max_length=200, verbose_name='treasurer lastname')),
                ('treasurer_firstname', models.CharField(help_text="First name of the payee committee's treasurer (from EXPN_CD.TRES_NAMF)", max_length=45, verbose_name='treasurer firstname')),
                ('treasurer_name_suffix', models.CharField(blank=True, help_text="Name suffix of the payee committee's treasurer (from EXPN_CD.TRES_NAMS)", max_length=10, verbose_name='treasurer name suffix')),
                ('treasurer_city', models.CharField(blank=True, help_text="City of the payee committee's treasurer (from EXPN_CD.TRES_CITY)", max_length=30, verbose_name='treasurer city')),
                ('treasurer_state', models.CharField(blank=True, help_text="State of the payee committee's treasurer (from EXPN_CD.TRES_ST)", max_length=2, verbose_name='treasurer state')),
                ('treasurer_zip', models.CharField(blank=True, help_text="Zip code (usually zip5, sometimes zip9) of the payee committee's treasurer (from EXPN_CD.TRES_ZIP4)", max_length=10, verbose_name='treasurer zip')),
                ('payment_code', models.CharField(blank=True, choices=[('CMP', 'Campaign paraphernalia/miscellaneous'), ('CNS', 'Campaign consultants'), ('CTB', 'Contribution (if nonmonetary, explain)*'), ('CVC', 'Civic donations'), ('FIL', 'Candidate filing/ballot feeds'), ('FND', 'Fundraising events'), ('IKD', 'In-kind contribution (nonmonetary)'), ('IND', 'Independent expenditure supporting/opposing others (explain)*'), ('LEG', 'Legal defense'), ('LIT', 'Campaign literature and mailings'), ('LON', 'Loan'), ('MBR', 'Member communications'), ('MON', 'Monetary contribution'), ('MTG', 'Meetings and appearances'), ('OFC', 'Office expenses'), ('PET', 'Petition circulating'), ('PHO', 'Phone banks'), ('POL', 'Polling and survey research'), ('POS', 'Postage, delivery and messenger services'), ('PRO', 'Professional services (legal, accounting)'), ('PRT', 'Print ads'), ('RAD', 'Radio airtime and production costs'), ('RFD', 'Returned contributions'), ('SAL', 'Campaign workers salaries'), ('TEL', 'T.V. or cable airtime and production costs'), ('TRC', 'Candidate travel, lodging and meals (explain)'), ('TRS', 'Staff/spouse travel, lodging and meals (explain)'), ('TSF', 'Transfer between committees of the same candidate/sponsor'), ('VOT', 'Voter registration'), ('WEB', 'Information technology costs (internet, e-mail)'), ('???', 'Unknown value')], help_text='Code describing the payment (from EXPN_CD.EXPN_CODE)', max_length=3, verbose_name='payment code')),
                ('payment_description', models.CharField(blank=True, help_text='Purpose of payment and/or description/explanation (from EXPN_CD.EXPN_DSCR)', max_length=400, verbose_name='payment description')),
                ('amount', models.DecimalField(decimal_places=2, help_text='Amount paid to the payee in the period covered by the filing (from EXPN_CD.AMOUNT)', max_digits=14, verbose_name='amount')),
                ('cumulative_ytd_amount', models.DecimalField(decimal_places=2, help_text="Cumulative year-to-date amount given or spent by the filer in support or opposition of the candidate or ballot measure as of the Form 460's filing date (from EXPN_CD.CUM_YTD)", max_digits=14, null=True)),
                ('expense_date', models.DateField(help_text='Date or expense (from EXPN_CD.EXPN_DATE)', null=True, verbose_name='expense date')),
                ('check_number', models.CharField(blank=True, help_text='Optional check number for the payment made by the campaign filer (from EXPN_CD.EXPN_CHKNO)', max_length=20, verbose_name='expense check number')),
                ('support_oppose_code', models.CharField(blank=True, choices=[('S', 'Support'), ('O', 'Oppose'), ('?', 'Unknown value')], help_text='If applicable, code indicating whether the payment went toward supporting or opposing a candidate/ballot measure (from EXPN_CD.SUP_OPP_CD)', max_length=1, verbose_name='support oppose code')),
                ('ballot_measure_jurisdiction', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a ballot measure, the jurisdiction subject to the ballot measure (from EXPN_CD.BAL_JURIS)', max_length=40, verbose_name='ballot measure jurisdiction')),
                ('ballot_measure_name', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a ballot measure, name of the ballot measure (from EXPN_CD.BAL_NAME  or EXPN_CD.CAND_NAML)', max_length=200, verbose_name='ballot measure name')),
                ('ballot_measure_num', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a ballot measure, ballot number or letter (from EXPN_CD.BAL_NUM)', max_length=7, verbose_name='ballot measure number')),
                ('candidate_title', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, name title of the candidate (from EXPN_CD.CAND_NAMT)', max_length=10, verbose_name='candidate title')),
                ('candidate_lastname', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, last name of the candidate or business name (from EXPN_CD.CAND_NAML)', max_length=200, verbose_name='candidate lastname')),
                ('candidate_firstname', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, first name of the candidate (from EXPN_CD.CAND_NAMF)', max_length=45, verbose_name='candidate firstname')),
                ('candidate_name_suffix', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate, name suffix of the candidate (from EXPN_CD.CAND_NAMS)', max_length=10, verbose_name='candidate name suffix')),
                ('candidate_jurisdiction_code', models.CharField(blank=True, choices=[('ASM', 'Assembly District'), ('BOE', 'Board of Equalization District'), ('CIT', 'City'), ('CTY', 'County'), ('LOC', 'Local'), ('OTH', 'Other'), ('SEN', 'Senate District'), ('STW', 'Statewide'), ('???', 'Statewide')], help_text='If the payment went toward supporting/opposing a candidate,code indicating the jurisdiction of the office (from EXPN_CD.JURIS_CD)', max_length=3, verbose_name='candidate jurisdiction')),
                ('candidate_jurisdiction_description', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a county, city or local candidate, full description of the office (from EXPN_CD.JURIS_DSCR)', max_length=40, verbose_name='candidate jurisdiciton description')),
                ('candidate_district', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate,for state senate, assembly or local board of education, thedistrict of the office (from EXPN_CD.DIST_NO)', max_length=3, verbose_name='candidate district')),
                ('office_sought_held', models.CharField(blank=True, choices=[('S', 'SOUGHT'), ('H', 'HELD')], help_text='If the payment went toward supporting/opposing a candidate,code indicating if the candidate is seeking or currently holds the office (from EXPN_CD.OFF_S_H_CD)', max_length=1, verbose_name='office sought or held')),
                ('office_code', models.CharField(blank=True, choices=[('APP', 'State Appellate Court Justice'), ('ASM', 'State Assembly Person'), ('ASR', 'Assessor'), ('ATT', 'Attorney General'), ('BED', 'Board of Education'), ('BOE', 'Board of Equalization Member'), ('BSU', 'Board of Supervisors'), ('CAT', 'City Attorney'), ('CCB', 'Community College Board'), ('CCM', 'City Council Member'), ('CON', 'State Controller'), ('COU', 'County Counsel'), ('CSU', 'County Supervisor'), ('CTR', 'Local Controller'), ('DAT', 'District Attorney'), ('GOV', 'Governor'), ('INS', 'Insurance Commissioner'), ('LTG', 'Lieutenant Governor'), ('MAY', 'Mayor'), ('OTH', 'Other'), ('PDR', 'Public Defender'), ('PER', 'Public Employees Retirement System'), ('PLN', 'Planning Commissioner'), ('SCJ', 'Superior Court Judge'), ('SEN', 'State Senator'), ('SHC', 'Sheriff-Coroner'), ('SOS', 'Secretary of State'), ('SPM', 'Supreme Court Justice'), ('SUP', 'Superintendent of Public Instruction'), ('TRE', 'State Treasurer'), ('TRS', 'Local Treasurer'), ('???', 'Unknown value')], help_text='If the payment went toward supporting/opposing a candidate,code describing the office (from EXPN_CD.OFFICE_CD)', max_length=3, verbose_name='office code')),
                ('office_description', models.CharField(blank=True, help_text='If the payment went toward supporting/opposing a candidate,description of the office (from EXPN_CD.OFFIC_DSCR)', max_length=40, verbose_name='office description')),
                ('transaction_id', models.CharField(help_text='Identifies a unique transaction across versions of the a given Form 460 filing (from EXPN_CD.TRAN_ID)', max_length=20, verbose_name='transaction id')),
                ('memo_reference_number', models.CharField(blank=True, help_text="A value assigned by the filer which refers to the item'sfootnote in the TEXT_MEMO_CD table (from EXPN_CD.MEMO_REFNO)", max_length=20, verbose_name='memo reference number')),
                ('filing_version', models.ForeignKey(help_text='Foreign key referring to the version of the Form 461 that includes the payment made', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='part_5_items', to='calaccess_processed_filings.Form461FilingVersion')),
            ],
            options={
                'verbose_name': 'Form 461 (Campaign Disclosure) Part 5 item version',
            },
        ),
        migrations.AlterUniqueTogether(
            name='form461part5itemversion',
            unique_together={('filing_version', 'line_item')},
        ),
        migrations.AlterIndexTogether(
            name='form461part5itemversion',
            index_together={('filing_version', 'line_item')},
        ),
        migrations.AlterUniqueTogether(
            name='form461part5item',
            unique_together={('filing', 'line_item')},
        ),
    ]



# Generated by Django 3.2 on 2021-04-26 19:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0010_auto_20180511_0001'),
    ]

    operations = [
        migrations.AlterField(
            model_name='form501filing',
            name='accepted_limit',
            field=models.BooleanField(help_text='Indicates if either the "I accept the voluntary expenditure ceiling" or "I do not accept the voluntary expenditure" box is checked (from F501_502_CD.ACCEPT_LIMIT_YN)', null=True),
        ),
        migrations.AlterField(
            model_name='form501filingversion',
            name='accepted_limit',
            field=models.BooleanField(help_text='Indicates if either the "I accept the voluntary expenditure ceiling" or "I do not accept the voluntary expenditure" box is checked (from F501_502_CD.ACCEPT_LIMIT_YN)', null=True),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load OCD RetentionContest and related models with data scraped from CAL-ACCESS.
"""
import re
from opencivicdata.core.models import Membership
from opencivicdata.elections.models import RetentionContest
from calaccess_processed.management.commands import CalAccessCommand
from calaccess_processed_elections.proxies import (
    OCDPostProxy,
    OCDPersonProxy,
    ScrapedCandidateProxy,
    ScrapedIncumbentProxy,
    ScrapedPropositionProxy
)


class Command(CalAccessCommand):
    """
    Load OCD RetentionContest and related models with data scraped from CAL-ACCESS.
    """
    help = 'Load OCD RetentionContest and related models with data scraped from CAL-ACCESS'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)
        self.header('Loading Retention Contests')
        self.load()
        self.success("Done!")

    def create_contest(self, scraped_prop, ocd_elec):
        """
        Create an OCD RetentionContest object derived from a ScrapedProposition.

        Return a RetentionContest object.
        """
        if scraped_prop.name == '2003 RECALL QUESTION':
            # look up most recently scraped record for Gov. Gray Davis
            incumbent = ScrapedCandidateProxy.objects.filter(
                name='DAVIS, GRAY',
                office_name__contains='GOVERNOR',
            ).latest('created')
        else:
            # extract the office name from the prop name
            office = [p.strip().replace("DISTRICT ", "") for p in scraped_prop.name.split("-") if 'DISTRICT' in p][0]
            session = re.search(r'\d{4}', scraped_prop.election.name).group()
            try:
                # look up the most recent scraped incumbent in the office
                incumbent = ScrapedIncumbentProxy.objects.filter(
                    office_name__contains=office,
                    session__lt=session
                )[0]
            except IndexError:
                raise Exception("Unknown Incumbent in %s." % scraped_prop.name)

        # get or create person and post objects
        person = OCDPersonProxy.objects.get_or_create_from_calaccess(
            incumbent.parsed_name,
            candidate_filer_id=incumbent.scraped_id
        )[0]
        post = OCDPostProxy.objects.get_or_create_by_name(incumbent.office_name)[0]

        # get or create membership object
        membership = Membership.objects.get_or_create(
            person=person,
            post=post,
            role=post.role,
            organization=post.organization,
            person_name=person.name,
        )[0]

        # set the start_date and end_date for Governor Gray Davis
        if scraped_prop.name == '2003 RECALL QUESTION':
            membership.start_date = '1999'
            membership.end_date = '2003'
            membership.save()

        # create the retention contest
        return RetentionContest.objects.create(
            election=ocd_elec,
            division=post.division,
            name=scraped_prop.name,
            membership=membership,
        )

    def load(self):
        """
        Load OCD ballot measure-related models with data scraped from CAL-ACCESS website.
        """
        object_list = ScrapedPropositionProxy.objects.filter(name__icontains='RECALL')
        for scraped_prop in object_list:
            ocd_election = scraped_prop.election_proxy.get_ocd_election()
            try:
                # Try getting the contest using scraped_id
                ocd_contest = ocd_election.retentioncontests.get(
                    identifiers__scheme='calaccess_measure_id',
                    identifiers__identifier=scraped_prop.scraped_id,
                )
            except RetentionContest.DoesNotExist:
                # If not there, create one
                ocd_contest = self.create_contest(scraped_prop, ocd_election)
                # Add the options
                ocd_contest.options.create(text='yes')
                ocd_contest.options.create(text='no')
                # Add the identifiers
                ocd_contest.identifiers.create(
                    scheme='calaccess_measure_id',
                    identifier=scraped_prop.scraped_id,
                )
                if self.verbosity > 2:
                    self.log(
                        'Created new {0}: {1}'.format(
                            ocd_contest._meta.object_name,
                            ocd_contest,
                        )
                    )
            else:
                # If the contest already exists, make sure the name is up-to-date
                # because they could change on subsequent scrapes of the SoS website
                if ocd_contest.name != scraped_prop.name:
                    ocd_contest.name = scraped_prop.name
                    ocd_contest.save()

            # Update or create the Contest source
            ocd_contest.sources.update_or_create(
                url=scraped_prop.url,
                note='Last scraped on {dt:%Y-%m-%d}'.format(
                    dt=scraped_prop.last_modified,
                )
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base classes for custom management commands.
"""
from __future__ import unicode_literals
import logging
from django.core.management import call_command
from calaccess_processed_elections.proxies import OCDDivisionProxy
from calaccess_processed.management.commands import CalAccessCommand
logger = logging.getLogger(__name__)


class LoadOCDElectionsBase(CalAccessCommand):
    """
    Base class for custom management commands that load the OCD Election model.
    """
    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(LoadOCDElectionsBase, self).handle(*args, **options)

        # Verify that OCD divisions have been loaded
        try:
            OCDDivisionProxy.objects.california()
        except OCDDivisionProxy.DoesNotExist:
            if self.verbosity > 2:
                self.log(' CA state division missing. Loading all U.S. divisions')
            call_command('loaddivisions', 'us')

    def load_from_proxy(self, proxy):
        """
        Load OCD Election from scraped proxy model.
        """
        # Loop through all elections in the proxy
        for scraped_election in proxy.objects.all():

            # Log each one as we go
            logger.debug("Loading from {}".format(scraped_election))

            # Get or create an election record
            ocd_election, ocd_created = scraped_election.get_or_create_ocd_election()

            # If we made a new one, log it out
            if ocd_created:
                logger.debug("Created {}".format(ocd_election))
                if self.verbosity > 1:
                    self.log(' Created new Election: {}'.format(ocd_election))

            # Whether Election is new or not, update EventSource
            ocd_election.sources.update_or_create(
                url=scraped_election.url,
                note='Last scraped on {:%Y-%m-%d}'.format(scraped_election.last_modified)
            )
